#ifndef _pdcp_configurations_H
#define _pdcp_configurations_H

#include <sys/types.h>
#include "pdcp_low_level.h"


struct pdcp_channel_configuration{
	uint16_t type;
	uint16_t profile;
	uint8_t channel_index;
	uint16_t owner_vid;
	uint16_t owner_pid;
	uint16_t owner_snum;
	uint16_t owner_type;
	uint16_t owner_profile;
};
typedef struct pdcp_channel_configuration pdcp_channel_configuration;

struct pdcp_channel_link_configuration{
	pdcp_channel_configuration input;
	pdcp_channel_configuration output;
	int implemented; // used in the code to indicate if this link has been attempted made.
};
typedef struct pdcp_channel_link_configuration pdcp_channel_link_configuration;

struct pdcp_configuration{
	uint8_t n_links;
	pdcp_channel_link_configuration* links;
};
typedef struct pdcp_configuration pdcp_configuration;

typedef enum confcode_t{
CONFCODE_NONE,
CONFCODE_N_OF_CONFIGURATIONS,
CONFCODE_CONFIGURATION,
CONFCODE_N_OF_LINKS,
CONFCODE_CHANNEL_LINK,
CONFCODE_INPUT_CHANNEL,
CONFCODE_OUTPUT_CHANNEL,
CONFCODE_CHANNEL_TYPE,
CONFCODE_CHANNEL_PROFILE,
CONFCODE_CHANNEL_INDEX,
CONFCODE_OWNER_VID,
CONFCODE_OWNER_PID,
CONFCODE_OWNER_SNUM,
CONFCODE_OWNER_TYPE,
CONFCODE_OWNER_PROFILE,
CONFCODE_END = 255,
} confcode_t;


void save_configuration(const pdcp_system_info* system_info);

// do not free return value
pdcp_configuration* load_configuration();

// compare a configuration channel link struct and a channel link info struct.
int conf_links_equal(pdcp_channel_link_configuration*, pdcp_channel_link*);

// compare a configuration channel struct and a channel info struct.
int conf_input_channels_equal(pdcp_channel_configuration* channel_conf, pdcp_input_channel* channel_info);
int conf_output_channels_equal(pdcp_channel_configuration* channel_conf, pdcp_output_channel* channel_info);


#endif
