#ifndef PDCP_COMM_DRIVER_H
#define PDCP_COMM_DRIVER_H

#include "stdint.h"
#include "stddef.h"
#include "sys/types.h"
#include "pdcp_names.h"

extern const uint16_t MIN_PDCP_UART_MSG_SIZE;

struct uart_msg_struct {
	uint16_t msg_id;
	uint8_t ref_id;
	uint16_t payload_size; 
	uint8_t* payload; // excluding CRC16
};

typedef struct uart_msg_struct uart_msg_struct;

struct uart_msg_array {
	uint8_t* array;
	size_t length;
};

typedef struct uart_msg_array uart_msg_array;

struct uart_conn{
	int read_fd;
	int write_fd;
	int message_is_ready;
	uint16_t payload_size_ready;
	uint8_t ref_id_ready;
};

typedef struct uart_conn uart_conn;



// run before use. out_conn will be set.
void driver_init(const char* read_portname, const char* write_portname, uart_conn* out_conn);

ssize_t driver_send_msg(uart_conn* conn, const uart_msg_struct* in);

// non-zero return value means new message is ready. return_code is enum
//return_code driver_check_for_msg(uart_conn* conn, uart_msg_struct* out);

// wrapper for driver_check_for_msg, blocks until return_code is non-zero
return_code driver_blocking_read(uart_conn* conn, uart_msg_struct* out);

// allocates and initializes uart_msg_struct
uart_msg_struct* driver_new_msg_struct(uint16_t payload_size);

// deallocates a uart_msg_struct allocated with driver_new_msg_struct
void driver_free_msg_struct(uart_msg_struct* msg);

#endif
