
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include "pdcp_configurations.h"
#include "pdcp_malloc.h"
#include "pdcp_error.h"


const char* filename;

pdcp_configuration current_configuration;
int loaded = 0;


// mallocs!
return_code parse_configuration(const uint8_t* conf_array, const size_t conf_array_size, pdcp_configuration** out_confs, int* n_confs);
return_code stringify_configurations(uint8_t* conf_array, int conf_array_size, const pdcp_configuration* confs, const int n_confs);

ssize_t write_array_to_file(const uint8_t* array, const size_t size, const char* filename);

// out array only valid until next call to this function
ssize_t read_array_from_file(uint8_t** out_array, const char* filename);
void link_info_to_link_configuration(const pdcp_channel_link* link, pdcp_channel_link_configuration* link_conf);
void system_info_to_configuration(const pdcp_system_info* system_info, pdcp_configuration* configuration);
uint16_t parse_uint16(const uint8_t* buffer);
void stringify_uint16(const uint16_t value, uint8_t* buffer);
void get_next_configuration_entry(uint8_t* label, uint16_t* value, const uint8_t* conf_array, int* array_index);
void write_configuration_entry(const confcode_t label, const uint16_t value, uint8_t* target, int* target_index);
size_t calculate_conf_array_size(const pdcp_configuration* conf);
int configuration_array_is_valid(const uint8_t* conf_array, int conf_array_size);


/*void pdcp_configurations_init(const char* m_filename){*/
/*	filename = m_filename;*/
/*	*/
/*	//load_configuration();*/
/*}*/

uint16_t parse_uint16(const uint8_t* buffer){
	return buffer[0] + (buffer[1]<<8);
}

void stringify_uint16(const uint16_t value, uint8_t* buffer){
	buffer[0] = value & 0xFF;
	buffer[1] = value >> 8;
}

void get_next_configuration_entry(uint8_t* label, uint16_t* value, const uint8_t* conf_array, int* array_index){
	*label = conf_array[*array_index];
	*value = parse_uint16(&conf_array[*array_index + 1]);
	*array_index += 3;
}

int configuration_array_is_valid(const uint8_t* conf_array, int conf_array_size){
	int has_n_confs = 0;
	int has_end = 0;
	int i;
	for(i = 0; (i < conf_array_size) & !has_n_confs; i++){
		if(CONFCODE_N_OF_CONFIGURATIONS == conf_array[i])
			has_n_confs = 1;
	}
	if (i>0) i--;
	for(i; i < conf_array_size; i+=3){
		if(CONFCODE_END == conf_array[i])
			has_end = 1;
	}
	return (has_n_confs && has_end);
}

return_code parse_configuration(const uint8_t* conf_array, const size_t conf_array_size, pdcp_configuration** out_confs, int* n_confs){	
	// check for NULL
	if (out_confs == NULL || n_confs == NULL) return BAD_POINTER;
	if (!configuration_array_is_valid(conf_array, conf_array_size)) return NO_ACTION;
	// initialize
	pdcp_configuration* confs;
	*n_confs = 0;
	uint8_t label = CONFCODE_NONE;
	uint16_t value;
	
	
	// allocate configuration array
	int array_index = 0;
	while(conf_array[array_index] != CONFCODE_N_OF_CONFIGURATIONS) array_index++;
	get_next_configuration_entry(&label, &value, conf_array, &array_index);
	*n_confs = value;
	confs = pdcp_malloc(*n_confs * sizeof(pdcp_configuration));
	
	// parse
	pdcp_configuration* current_conf;
	pdcp_channel_link_configuration* current_link;
	int is_input; // state variable: input/output
	
	while (label != CONFCODE_END){
		get_next_configuration_entry(&label, &value, conf_array, &array_index);
		
/*		printf("label: %d\n", (unsigned)label);*/
		switch (label){
		case CONFCODE_CONFIGURATION:
			current_conf = &confs[value];
			break;
		case CONFCODE_N_OF_LINKS:
			current_conf->n_links = value;
/*			printf ("conf n channel links: %d\n", value);*/
			current_conf->links = (pdcp_channel_link_configuration*)pdcp_malloc(value* sizeof(pdcp_channel_link_configuration));
/*			printf("current_conf->links: %X\n", (unsigned)current_conf->links);*/
			break;
		case CONFCODE_CHANNEL_LINK:
/*			printf ("conf channel link index: %d\n", value);*/
			current_link = &(current_conf->links[value]);
			current_link->implemented = 0;
			break;
		case CONFCODE_INPUT_CHANNEL:
			is_input = 1;
			break;
		case CONFCODE_OUTPUT_CHANNEL:
			is_input = 0;
			break;
		case CONFCODE_CHANNEL_TYPE:
			if(is_input) 	current_link->input.type = value;
			else 		current_link->output.type = value;
			break;
		case CONFCODE_CHANNEL_PROFILE:
			if(is_input) 	current_link->input.profile = value; 
			else		current_link->output.profile = value;
			break;
		case CONFCODE_CHANNEL_INDEX:
			if(is_input) 	current_link->input.channel_index = (uint8_t)value; 
			else		current_link->output.channel_index = (uint8_t)value;
			break;
		case CONFCODE_OWNER_PID:
			if(is_input) 	current_link->input.owner_pid = value; 
			else		current_link->output.owner_pid = value;
			break;
		case CONFCODE_OWNER_VID:
			if(is_input)	current_link->input.owner_vid = value; 
			else		current_link->output.owner_vid = value;
			break;
		case CONFCODE_OWNER_SNUM:
			if(is_input)	current_link->input.owner_snum = value; 
			else		current_link->output.owner_snum = value;
			break;
		case CONFCODE_OWNER_TYPE:
			if(is_input)	current_link->input.owner_type = value; 
			else		current_link->output.owner_type = value;
			break;
		case CONFCODE_OWNER_PROFILE:
			if(is_input)	current_link->input.owner_profile = value; 
			else		current_link->output.owner_profile = value;
			break;
		default:
			break;
		}
	}	
	
	*out_confs = confs;
	return OUT_VAR_FILLED;
}

void write_configuration_entry(const confcode_t label, const uint16_t value, uint8_t* target, int* target_index){
	target[*target_index] = (uint8_t)label;
	stringify_uint16(value, &target[*target_index+1]);
	*target_index += 3;
}

return_code stringify_configurations(uint8_t* conf_array, int conf_array_size, const pdcp_configuration* confs, const int n_confs){
	if (conf_array == NULL || confs == NULL)
		return BAD_POINTER;
	
	if (conf_array_size < 3)
		return TOO_SMALL;
		
	int array_index = 0;
	write_configuration_entry(CONFCODE_N_OF_CONFIGURATIONS, n_confs, conf_array, &array_index);
	for (int conf_index = 0; conf_index < n_confs; conf_index++){
		if (array_index + 6 > conf_array_size) return TOO_SMALL;
		write_configuration_entry(CONFCODE_CONFIGURATION, conf_index, conf_array, &array_index);
		write_configuration_entry(CONFCODE_N_OF_LINKS, confs[conf_index].n_links, conf_array, &array_index);
		for(int link_index = 0; link_index < confs[conf_index].n_links; link_index++){
			if (array_index + 17*3 > (conf_array_size - array_index)) return TOO_SMALL;
			write_configuration_entry(CONFCODE_CHANNEL_LINK, link_index, conf_array, &array_index);
			for (int output = 0; output < 2; output++){
				pdcp_channel_configuration* current_channel = output ? &confs[conf_index].links[link_index].output : &confs[conf_index].links[link_index].input;
				if (output) write_configuration_entry(CONFCODE_OUTPUT_CHANNEL, 0, conf_array, &array_index);
				else		write_configuration_entry(CONFCODE_INPUT_CHANNEL, 0, conf_array, &array_index);
				
				write_configuration_entry(CONFCODE_CHANNEL_TYPE, current_channel->type, conf_array, &array_index);
				write_configuration_entry(CONFCODE_CHANNEL_PROFILE, current_channel->profile, conf_array, &array_index);
				write_configuration_entry(CONFCODE_CHANNEL_INDEX, current_channel->channel_index, conf_array, &array_index);
				write_configuration_entry(CONFCODE_OWNER_VID, current_channel->owner_vid, conf_array, &array_index);
				write_configuration_entry(CONFCODE_OWNER_PID, current_channel->owner_pid, conf_array, &array_index);
				write_configuration_entry(CONFCODE_OWNER_SNUM, current_channel->owner_snum, conf_array, &array_index);
				write_configuration_entry(CONFCODE_OWNER_TYPE, current_channel->owner_type, conf_array, &array_index);
				write_configuration_entry(CONFCODE_OWNER_PROFILE, current_channel->owner_profile, conf_array, &array_index);
			}
		}
	}
	if ((conf_array_size - array_index) < 3)
		return TOO_SMALL;
	write_configuration_entry(CONFCODE_END, 0, conf_array, &array_index);
	return OUT_VAR_FILLED;
}


ssize_t write_array_to_file(const uint8_t* array, const size_t size, const char* filename){
	int fd = open(filename, O_WRONLY|O_TRUNC); 
	
	ssize_t n_written_bytes = write(fd, array, size);
	
	close(fd);
	
	return n_written_bytes;
}

// out array only valid until next call to this function
ssize_t read_array_from_file(uint8_t** out_array, const char* filename){
	static uint8_t* array = NULL;
	int fd = open(filename, O_RDONLY);
	
	struct stat buf;
	int fstat_ret = fstat(fd, &buf);
	check_ret_value(fstat_ret, __LINE__);
	array = (uint8_t*)pdcp_realloc(array, buf.st_size);
	
	ssize_t n_read_bytes = read(fd, array, buf.st_size);
	
	close(fd);
	
	*out_array = &array[0];
	return n_read_bytes;
}


void link_info_to_link_configuration(const pdcp_channel_link* link, pdcp_channel_link_configuration* link_conf){
	
	
	link_conf->input.type = link->input_channel->type;
	link_conf->input.profile = link->input_channel->profile;
	link_conf->input.channel_index = link->input_channel->channel_index;
	link_conf->input.owner_vid = link->input_channel->owner->vendor_id;
	link_conf->input.owner_pid = link->input_channel->owner->product_id;
	link_conf->input.owner_snum = link->input_channel->owner->serial_num;
	link_conf->input.owner_type = link->input_channel->owner->type;
	link_conf->input.owner_profile = link->input_channel->owner->profile;
	
	link_conf->output.type = link->output_channel->type;
	link_conf->output.profile = link->output_channel->profile;
	link_conf->output.channel_index = link->output_channel->channel_index;
	link_conf->output.owner_vid = link->output_channel->owner->vendor_id;
	link_conf->output.owner_pid = link->output_channel->owner->product_id;
	link_conf->output.owner_snum = link->output_channel->owner->serial_num;
	link_conf->output.owner_type = link->output_channel->owner->type;
	link_conf->output.owner_profile = link->output_channel->owner->profile;
	
}

void system_info_to_configuration(const pdcp_system_info* system_info, pdcp_configuration* configuration){
	
/*	print_system_info(system_info);*/
	
	int conf_link_it = 0;
	for (int i = 0; i < configuration->n_links; i++){
		if(system_info->channel_links[i].link_status == LINK_STATUS_ACTIVE){
			link_info_to_link_configuration(&system_info->channel_links[i], &configuration->links[conf_link_it]);	
			conf_link_it++;
		}
	}
	configuration->n_links = conf_link_it;
}

size_t calculate_conf_array_size(const pdcp_configuration* conf){
	size_t size = 	3 + //n_confs
					3 + //conf
					3 + //n_links
					conf->n_links * ( 
					3 + //link
					3 + //input
					3 + //output
					15);//channel_info
	
	return size;
}

void save_configuration(const pdcp_system_info* system_info){

	current_configuration.n_links = system_info->n_channel_links;
	current_configuration.links = pdcp_realloc(current_configuration.links, system_info->n_channel_links * sizeof(pdcp_channel_link_configuration));
		
	uint8_t* conf_array = NULL;
	
	system_info_to_configuration(system_info, &current_configuration);
	
	size_t conf_array_size = calculate_conf_array_size(&current_configuration);
	conf_array = pdcp_malloc(conf_array_size * sizeof(uint8_t));
	
	
	return_code ret = stringify_configurations(conf_array, conf_array_size, &current_configuration, 1);
	
	while (TOO_SMALL == ret){
		conf_array_size *= 2;
		conf_array = pdcp_realloc(conf_array, conf_array_size * sizeof(uint8_t));
		ret = stringify_configurations(conf_array, conf_array_size, &current_configuration, 1);
	}
	
	ssize_t write_size = write_array_to_file(conf_array, conf_array_size, CONFIGURATIONS_PATH);
	check_ret_value(write_size, __LINE__);
	
	pdcp_free(conf_array);
	
}

pdcp_configuration* load_configuration(){	
	if(!loaded){
		loaded = 1;
		uint8_t* conf_array;
		ssize_t read_size = read_array_from_file(&conf_array, CONFIGURATIONS_PATH);
		check_ret_value(read_size, __LINE__);

		pdcp_configuration* temp_conf;
		int n_confs;
		
			
		return_code ret = parse_configuration(conf_array, read_size, &temp_conf, &n_confs);
		
		if(OUT_VAR_FILLED != ret){
			//printf("no configuration, ret: %d\n", ret);
			return NULL;
		}
		
		memcpy(&current_configuration, temp_conf, sizeof(pdcp_configuration));
		
		pdcp_free(temp_conf);
	}
	
	return &current_configuration;
}



int conf_links_equal(pdcp_channel_link_configuration* link_conf, pdcp_channel_link* link_info){
	
	return (conf_input_channels_equal(&link_conf->input, link_info->input_channel) && 
		conf_output_channels_equal(&link_conf->output, link_info->output_channel));
}

int conf_input_channels_equal(pdcp_channel_configuration* channel_conf, pdcp_input_channel* channel_info){
	return (	channel_conf->owner_vid == channel_info->owner->vendor_id &&
			channel_conf->owner_pid == channel_info->owner->product_id &&
			channel_conf->owner_snum == channel_info->owner->serial_num &&
			channel_conf->channel_index == channel_info->channel_index	);
}

int conf_output_channels_equal(pdcp_channel_configuration* channel_conf, pdcp_output_channel* channel_info){
	return (	channel_conf->owner_vid == channel_info->owner->vendor_id &&
			channel_conf->owner_pid == channel_info->owner->product_id &&
			channel_conf->owner_snum == channel_info->owner->serial_num &&
			channel_conf->channel_index == channel_info->channel_index	);
}
