#include "pdcp_test_helper.h"
#include "pdcp_driver.h"
#include "pdcp_low_level.h"
#include "pdcp_error.h"
#include "pdcp_malloc.h"
#include <stdint.h>
#include <stdio.h>
#include <unistd.h>

uart_conn conn;


return_code driver_struct_to_array(const uart_msg_struct* in, uart_msg_array* out);


void test_helper_init(uart_conn* new_conn){
	conn.read_fd = new_conn->read_fd;
}

void write_failure_response(uint16_t msg_id, uint8_t error_code){
	
	uart_msg_struct response = {
		.msg_id = msg_id,
		.ref_id = 13,
		.payload_size = 1,
		.payload = &error_code
	};
	
	uint8_t array[11];
	uart_msg_array response_array = {
		.length = 11,
		.array = array
	};
	driver_struct_to_array(&response, &response_array);
	
	ssize_t write_count = write(conn.read_fd, response_array.array, 11);
	check_ret_value(write_count, __LINE__);

}


void write_system_info_response(uint8_t n_devices, uint8_t n_input_channels, uint8_t n_output_channels, uint8_t n_channel_links){
	uint8_t response_payload[4] = {n_devices, n_input_channels, n_output_channels, n_channel_links};
	uart_msg_struct response = {
		.msg_id = MSG_ID_GET_SYSTEM_INFO_SUCCESS,
		.ref_id = 13,
		.payload_size = 4,
		.payload = response_payload
	};
	
	uint8_t array[14];
	uart_msg_array response_array = {
		.length = 14,
		.array = array
	};
	driver_struct_to_array(&response, &response_array);
	
	ssize_t write_count = write(conn.read_fd, response_array.array, 14);
	check_ret_value(write_count, __LINE__);
}


void write_device_info_response(uint8_t snum_lsb, uint8_t type_lsb, uint8_t profile_lsb){
	uint8_t response_payload[11] = {0x34, 0x12, 0x45, 0x23, snum_lsb, 0x34, 0x78, type_lsb, 0x56, profile_lsb, 0x67};
	uart_msg_struct response = {
		.msg_id = MSG_ID_GET_DEVICE_INFO_SUCCESS,
		.ref_id = 13,
		.payload_size = 11,
		.payload = response_payload
	};
	
	uint8_t array[21];
	uart_msg_array response_array = {
		.length = 21,
		.array = array
	};
	
	driver_struct_to_array(&response, &response_array);
	ssize_t write_count = write(conn.read_fd, response_array.array, 21);

	check_ret_value(write_count, __LINE__);
}
	
void write_input_channel_info_response(uint8_t snum_lsb, uint8_t channel_index, uint16_t type, uint16_t profile){
	uint8_t response_payload[11] = {0x34, 0x12, 0x45, 0x23, snum_lsb, 0x34, channel_index, type, type >> 8, profile, profile >> 8};
	uart_msg_struct response = {
		.msg_id = MSG_ID_GET_INPUT_CHANNEL_INFO_SUCCESS,
		.ref_id = 13,
		.payload_size = 11,
		.payload = response_payload
	};
	
	uint8_t array[21];
	uart_msg_array response_array = {
		.length = 21,
		.array = array
	};
	
	driver_struct_to_array(&response, &response_array);
	ssize_t write_count = write(conn.read_fd, response_array.array, 21);
	check_ret_value(write_count, __LINE__);
}


void write_output_channel_info_response(uint8_t snum_lsb, uint8_t channel_index, uint16_t type, uint16_t profile){

	uint8_t response_payload[11] = {0x34, 0x12, 0x45, 0x23, snum_lsb, 0x34, channel_index, type, type >> 8, profile, profile >> 8};
	uart_msg_struct response = {
		.msg_id = MSG_ID_GET_OUTPUT_CHANNEL_INFO_SUCCESS,
		.ref_id = 13,
		.payload_size = 11,
		.payload = response_payload
	};
	
	uint8_t array[21];
	uart_msg_array response_array = {
		.length = 21,
		.array = array
	};
	
	driver_struct_to_array(&response, &response_array);
	ssize_t write_count = write(conn.read_fd, response_array.array, 21);
	check_ret_value(write_count, __LINE__);
}


void write_channel_link_info_response(uint8_t out_snum_lsb, uint8_t out_channel_index, uint8_t in_snum_lsb, uint8_t in_channel_index, uint8_t link_status){
	uint8_t response_payload[15] = {0x34, 0x12, 0x45, 0x23, out_snum_lsb, 0x34, out_channel_index, 0x34, 0x12, 0x45, 0x23, in_snum_lsb, 0x34, in_channel_index, link_status};
	uart_msg_struct response = {
		.msg_id = MSG_ID_GET_CHANNEL_LINK_INFO_SUCCESS,
		.ref_id = 13,
		.payload_size = 15,
		.payload = response_payload
	};

	uint8_t array[25];
	uart_msg_array response_array = {
		.length = 25,
		.array = array
	};

	driver_struct_to_array(&response, &response_array);
	ssize_t write_count = write(conn.read_fd, response_array.array, 25);
	check_ret_value(write_count, __LINE__);
}


void write_get_parameter_response_value(size_t size, uint8_t* value){
	uart_msg_struct response = {
		.msg_id = MSG_ID_GET_DEVICE_PARAMETER_SUCCESS,
		.ref_id = 13,
		.payload_size = size,
		.payload = value
	};
	
	uint8_t* array = (uint8_t*)pdcp_malloc((size+10) * sizeof(uint8_t));
	uart_msg_array response_array = {
		.length = size+10,
		.array = array
	};

	driver_struct_to_array(&response, &response_array);
	
	ssize_t write_count = write(conn.read_fd, response_array.array, size+10);
	check_ret_value(write_count, __LINE__);
	
	pdcp_free(array);
}

void write_get_parameter_response(size_t size){
	uint8_t* response_payload = (uint8_t*)pdcp_malloc(size * sizeof(uint8_t));
	
	for(int i = 0; i < size; i++){
		response_payload[i] = 0xA0 + i;
	}
	
	write_get_parameter_response_value(size, response_payload);
	
	pdcp_free(response_payload);
}
	
void write_set_parameter_response(){
	uart_msg_struct response = {
		.msg_id = MSG_ID_SET_DEVICE_PARAMETER_SUCCESS,
		.ref_id = 13,
		.payload_size = 0,
		.payload = NULL
	};
	
	uint8_t array[8];
	uart_msg_array response_array = {
		.length = 8,
		.array = &array[0]
	};
	
	driver_struct_to_array(&response, &response_array);
	
	ssize_t write_count = write(conn.read_fd, response_array.array, 8);
	check_ret_value(write_count, __LINE__);
}

void write_configure_channel_link_response(uint8_t link_status){
	write_failure_response(MSG_ID_CONFIGURE_CHANNEL_LINK_SUCCESS, link_status);
}

// ssize_t read_into(char* filename, uint8_t* buffer, size_t count){
	// int fd = open(filename, O_RDONLY);
	
	// ssize_t ret = read(fd, buffer, count);
	
	// close(fd);
	
	// return ret;
// }
