#include "pdcp_driver.h"
#include "pdcp_low_level.h"
#include "pdcp_configurations.h"
#include "pdcp_test_helper.h"
#include "pdcp_error.h"
#include "unity.h"
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

uart_conn conn;

void setUp(void){
	conn.read_fd = open(READ_PATH, O_RDWR | O_NOCTTY | O_SYNC | O_TRUNC);
	if(conn.read_fd < 0){
		char error_msg[25];
		sprintf(error_msg, "open failed: %d", errno);
	}
	conn.write_fd = open(WRITE_PATH, O_RDWR | O_NOCTTY | O_SYNC | O_TRUNC);
	if(conn.read_fd < 0){
		char error_msg[25];
		sprintf(error_msg, "open failed: %d", errno);
		TEST_FAIL_MESSAGE(error_msg);
	}
	test_helper_init(&conn);
	
}

void tearDown(void){
	close(conn.read_fd);
}


void test_parseConfiguration(){
	uint8_t configuration[150] = {
		CONFCODE_N_OF_CONFIGURATIONS, 2, 0,
		CONFCODE_CONFIGURATION, 0, 0,
		CONFCODE_N_OF_LINKS, 1, 0,
		CONFCODE_CHANNEL_LINK, 0, 0, 
		
		CONFCODE_INPUT_CHANNEL, 0, 0,
		CONFCODE_CHANNEL_TYPE, 0x76, 0x98,
		CONFCODE_CHANNEL_PROFILE, 0x65, 0x87,
		CONFCODE_CHANNEL_INDEX, 0x0, 0x0,
		CONFCODE_OWNER_VID, 0x34, 0x12,
		CONFCODE_OWNER_PID, 0x45, 0x23,
		CONFCODE_OWNER_SNUM, 0x56, 0x34,
		CONFCODE_OWNER_TYPE, 0x54, 0x76,
		CONFCODE_OWNER_PROFILE, 0x43, 0x65,
		
		CONFCODE_OUTPUT_CHANNEL, 0, 0,
		CONFCODE_CHANNEL_TYPE, 0x76, 0x98,
		CONFCODE_CHANNEL_PROFILE, 0x65, 0x87,
		CONFCODE_CHANNEL_INDEX, 0, 0,
		CONFCODE_OWNER_VID, 0x34, 0x12,
		CONFCODE_OWNER_PID, 0x45, 0x23,
		CONFCODE_OWNER_SNUM, 0x57, 0x34,
		CONFCODE_OWNER_TYPE, 0x54, 0x76,
		CONFCODE_OWNER_PROFILE, 0x43, 0x65,
		
		CONFCODE_CONFIGURATION, 1, 0,
		CONFCODE_N_OF_LINKS, 1, 0,
		CONFCODE_CHANNEL_LINK, 0, 0, 
		
		CONFCODE_INPUT_CHANNEL, 0, 0,
		CONFCODE_CHANNEL_TYPE, 0x76, 0x98,
		CONFCODE_CHANNEL_PROFILE, 0x65, 0x87,
		CONFCODE_CHANNEL_INDEX, 1, 0,
		CONFCODE_OWNER_VID, 0x34, 0x12,
		CONFCODE_OWNER_PID, 0x45, 0x23,
		CONFCODE_OWNER_SNUM, 0x57, 0x34,
		CONFCODE_OWNER_TYPE, 0x54, 0x76,
		CONFCODE_OWNER_PROFILE, 0x43, 0x65,
		
		CONFCODE_OUTPUT_CHANNEL, 0, 0,
		CONFCODE_CHANNEL_TYPE, 0x76, 0x98,
		CONFCODE_CHANNEL_PROFILE, 0x65, 0x87,
		CONFCODE_CHANNEL_INDEX, 1, 0,
		CONFCODE_OWNER_VID, 0x34, 0x12,
		CONFCODE_OWNER_PID, 0x45, 0x23,
		CONFCODE_OWNER_SNUM, 0x56, 0x34,
		CONFCODE_OWNER_TYPE, 0x54, 0x76,
		CONFCODE_OWNER_PROFILE, 0x43, 0x65,
		
		CONFCODE_END, 0, 0
	};
	
/*	pdcp_configuration confs[2];*/
	pdcp_configuration* confs = NULL;
	
	int n_confs;
	return_code ret = parse_configuration(&configuration[0], 150, NULL, &n_confs);
	TEST_ASSERT_EQUAL(BAD_POINTER, ret);
/*	ret = parse_configuration(&configuration[0], &confs[0], 1);*/
/*	TEST_ASSERT_EQUAL(TOO_SMALL, ret);*/
	ret = parse_configuration(&configuration[0], 150, &confs, &n_confs);
	TEST_ASSERT_EQUAL(OUT_VAR_FILLED, ret);
	TEST_ASSERT_EQUAL(2, n_confs);
	
	TEST_ASSERT_EQUAL(0x9876, confs[0].links[0].input.type);
	TEST_ASSERT_EQUAL_HEX16(0x8765, confs[0].links[0].input.profile);
	TEST_ASSERT_EQUAL_HEX8(0, confs[0].links[0].input.channel_index);
	TEST_ASSERT_EQUAL_HEX16(0x1234, confs[0].links[0].input.owner_vid);
	TEST_ASSERT_EQUAL_HEX16(0x2345, confs[0].links[0].input.owner_pid);
	TEST_ASSERT_EQUAL_HEX16(0x3456, confs[0].links[0].input.owner_snum);
	TEST_ASSERT_EQUAL_HEX16(0x7654, confs[0].links[0].input.owner_type);
	TEST_ASSERT_EQUAL_HEX16(0x6543, confs[0].links[0].input.owner_profile);
	
	TEST_ASSERT_EQUAL_HEX16(0x9876, confs[0].links[0].output.type);
	TEST_ASSERT_EQUAL_HEX16(0x8765, confs[0].links[0].output.profile);
	TEST_ASSERT_EQUAL_HEX8(0, confs[0].links[0].output.channel_index);
	TEST_ASSERT_EQUAL_HEX16(0x1234, confs[0].links[0].output.owner_vid);
	TEST_ASSERT_EQUAL_HEX16(0x2345, confs[0].links[0].output.owner_pid);
	TEST_ASSERT_EQUAL_HEX16(0x3457, confs[0].links[0].output.owner_snum);
	TEST_ASSERT_EQUAL_HEX16(0x7654, confs[0].links[0].output.owner_type);
	TEST_ASSERT_EQUAL_HEX16(0x6543, confs[0].links[0].output.owner_profile);
	
	TEST_ASSERT_EQUAL_HEX16(0x9876, confs[1].links[0].input.type);
	TEST_ASSERT_EQUAL_HEX16(0x8765, confs[1].links[0].input.profile);
	TEST_ASSERT_EQUAL_HEX8(1, confs[1].links[0].input.channel_index);
	TEST_ASSERT_EQUAL_HEX16(0x1234, confs[1].links[0].input.owner_vid);
	TEST_ASSERT_EQUAL_HEX16(0x2345, confs[1].links[0].input.owner_pid);
	TEST_ASSERT_EQUAL_HEX16(0x3457, confs[1].links[0].input.owner_snum);
	TEST_ASSERT_EQUAL_HEX16(0x7654, confs[1].links[0].input.owner_type);
	TEST_ASSERT_EQUAL_HEX16(0x6543, confs[1].links[0].input.owner_profile);
	
	TEST_ASSERT_EQUAL_HEX16(0x9876, confs[1].links[0].output.type);
	TEST_ASSERT_EQUAL_HEX16(0x8765, confs[1].links[0].output.profile);
	TEST_ASSERT_EQUAL_HEX8(1, confs[1].links[0].output.channel_index);
	TEST_ASSERT_EQUAL_HEX16(0x1234, confs[1].links[0].output.owner_vid);
	TEST_ASSERT_EQUAL_HEX16(0x2345, confs[1].links[0].output.owner_pid);
	TEST_ASSERT_EQUAL_HEX16(0x3456, confs[1].links[0].output.owner_snum);
	TEST_ASSERT_EQUAL_HEX16(0x7654, confs[1].links[0].output.owner_type);
	TEST_ASSERT_EQUAL_HEX16(0x6543, confs[1].links[0].output.owner_profile);


	
}


void test_stringifyConfiguration(){
	pdcp_channel_link_configuration links0;
	pdcp_channel_link_configuration links1;
	pdcp_configuration confs[] = {
		{
			.n_links = 1,
			.links = &links0
		},
		{
			.n_links = 1,
			.links = &links1
		}
	};
	confs[0].links[0].input.type = 0x9876;
	confs[0].links[0].input.profile = 0x8765;
	confs[0].links[0].input.channel_index = 0;
	confs[0].links[0].input.owner_vid = 0x1234;
	confs[0].links[0].input.owner_pid = 0x2345;
	confs[0].links[0].input.owner_snum = 0x3456;
	confs[0].links[0].input.owner_type = 0x7654;
	confs[0].links[0].input.owner_profile = 0x6543;
	
	confs[0].links[0].output.type = 0x9876;
	confs[0].links[0].output.profile = 0x8765;
	confs[0].links[0].output.channel_index = 0;
	confs[0].links[0].output.owner_vid = 0x1234;
	confs[0].links[0].output.owner_pid = 0x2345;
	confs[0].links[0].output.owner_snum = 0x3457;
	confs[0].links[0].output.owner_type = 0x7654;
	confs[0].links[0].output.owner_profile = 0x6543;
	
	confs[1].links[0].input.type = 0x9876;
	confs[1].links[0].input.profile = 0x8765;
	confs[1].links[0].input.channel_index = 1;
	confs[1].links[0].input.owner_vid = 0x1234;
	confs[1].links[0].input.owner_pid = 0x2345;
	confs[1].links[0].input.owner_snum = 0x3457;
	confs[1].links[0].input.owner_type = 0x7654;
	confs[1].links[0].input.owner_profile = 0x6543;
	
	confs[1].links[0].output.type = 0x9876;
	confs[1].links[0].output.profile = 0x8765;
	confs[1].links[0].output.channel_index = 1;
	confs[1].links[0].output.owner_vid = 0x1234;
	confs[1].links[0].output.owner_pid = 0x2345;
	confs[1].links[0].output.owner_snum = 0x3456;
	confs[1].links[0].output.owner_type = 0x7654;
	confs[1].links[0].output.owner_profile = 0x6543;
	
	uint8_t conf_string[200];
	
	return_code ret = stringify_configurations(&conf_string[0], 20, &confs[0], 2);
	TEST_ASSERT_EQUAL(TOO_SMALL, ret);
	ret = stringify_configurations(&conf_string[0], 200, &confs[0], 2);
	TEST_ASSERT_EQUAL(OUT_VAR_FILLED, ret);
	
	uint8_t expected_string[] = {
		CONFCODE_N_OF_CONFIGURATIONS, 2, 0,
		CONFCODE_CONFIGURATION, 0, 0,
		CONFCODE_N_OF_LINKS, 1, 0,
		CONFCODE_CHANNEL_LINK, 0, 0, 
		
		CONFCODE_INPUT_CHANNEL, 0, 0,
		CONFCODE_CHANNEL_TYPE, 0x76, 0x98,
		CONFCODE_CHANNEL_PROFILE, 0x65, 0x87,
		CONFCODE_CHANNEL_INDEX, 0x0, 0x0,
		CONFCODE_OWNER_VID, 0x34, 0x12,
		CONFCODE_OWNER_PID, 0x45, 0x23,
		CONFCODE_OWNER_SNUM, 0x56, 0x34,
		CONFCODE_OWNER_TYPE, 0x54, 0x76,
		CONFCODE_OWNER_PROFILE, 0x43, 0x65,
		
		CONFCODE_OUTPUT_CHANNEL, 0, 0,
		CONFCODE_CHANNEL_TYPE, 0x76, 0x98,
		CONFCODE_CHANNEL_PROFILE, 0x65, 0x87,
		CONFCODE_CHANNEL_INDEX, 0, 0,
		CONFCODE_OWNER_VID, 0x34, 0x12,
		CONFCODE_OWNER_PID, 0x45, 0x23,
		CONFCODE_OWNER_SNUM, 0x57, 0x34,
		CONFCODE_OWNER_TYPE, 0x54, 0x76,
		CONFCODE_OWNER_PROFILE, 0x43, 0x65,
		
		CONFCODE_CONFIGURATION, 1, 0,
		CONFCODE_N_OF_LINKS, 1, 0,
		CONFCODE_CHANNEL_LINK, 0, 0, 
		
		CONFCODE_INPUT_CHANNEL, 0, 0,
		CONFCODE_CHANNEL_TYPE, 0x76, 0x98,
		CONFCODE_CHANNEL_PROFILE, 0x65, 0x87,
		CONFCODE_CHANNEL_INDEX, 1, 0,
		CONFCODE_OWNER_VID, 0x34, 0x12,
		CONFCODE_OWNER_PID, 0x45, 0x23,
		CONFCODE_OWNER_SNUM, 0x57, 0x34,
		CONFCODE_OWNER_TYPE, 0x54, 0x76,
		CONFCODE_OWNER_PROFILE, 0x43, 0x65,
		
		CONFCODE_OUTPUT_CHANNEL, 0, 0,
		CONFCODE_CHANNEL_TYPE, 0x76, 0x98,
		CONFCODE_CHANNEL_PROFILE, 0x65, 0x87,
		CONFCODE_CHANNEL_INDEX, 1, 0,
		CONFCODE_OWNER_VID, 0x34, 0x12,
		CONFCODE_OWNER_PID, 0x45, 0x23,
		CONFCODE_OWNER_SNUM, 0x56, 0x34,
		CONFCODE_OWNER_TYPE, 0x54, 0x76,
		CONFCODE_OWNER_PROFILE, 0x43, 0x65,
		
		CONFCODE_END, 0, 0
	};
	
	TEST_ASSERT_EQUAL_HEX8_ARRAY(expected_string, conf_string, 123);
}

void test_writeReadArrayToFromFile(){
	
	uint8_t array[] = {1,2,3,4,5,6,7,8,9,10,255,254,253,100,101,127};
	uint8_t* read_array;
	
	ssize_t n_written_bytes = write_array_to_file(&array[0], 16, "configurations.file");
	check_ret_value(n_written_bytes, __LINE__);
	
	ssize_t n_read_bytes = read_array_from_file(&read_array, "configurations.file");
	check_ret_value(n_read_bytes, __LINE__);
	
	TEST_ASSERT_EQUAL(16, n_written_bytes);
	TEST_ASSERT_EQUAL(16, n_read_bytes);
	TEST_ASSERT_EQUAL_HEX8_ARRAY(&array[0], &read_array[0], 16);
}

void test_systemInfoToConfiguration(){
	write_system_info_response(2,3,2,3);
	write_device_info_response(0x56, 0x78, 0x89);
	write_device_info_response(0x57, 0x79, 0x8A);
	write_input_channel_info_response(0x56, 2, 0x5678, 0x6789);
	write_input_channel_info_response(0x57, 1, 0x5678, 0x6789);
	write_input_channel_info_response(0x57, 3, 0x5678, 0x6789);
	write_output_channel_info_response(0x56, 1, 0x5678, 0x6789);
	write_output_channel_info_response(0x57, 2, 0x5678, 0x6789);
	write_channel_link_info_response(0x56, 1, 0x57, 1, 1);
	write_channel_link_info_response(0x57, 2, 0x56, 2, 1);
	write_channel_link_info_response(0x56, 1, 0x57, 3, 1);
	
	lseek(conn.read_fd, 0, SEEK_SET);
	
	pdcp_system_info system_info;
	low_level_init(&system_info);
	
	system_info.channel_links[0].link_status=LINK_STATUS_ACTIVE;
	system_info.channel_links[1].link_status=LINK_STATUS_ACTIVE;
	system_info.channel_links[2].link_status=LINK_STATUS_INACTIVE;

	pdcp_channel_link_configuration links[3];
	pdcp_configuration conf = {
		.n_links = 3,
		.links = &links[0]
	};
	
	system_info_to_configuration(&system_info, &conf);
	
	TEST_ASSERT_EQUAL(2, conf.n_links);
	
	TEST_ASSERT_EQUAL_HEX16(0x5678, conf.links[0].input.type);
	TEST_ASSERT_EQUAL_HEX16(0x6789, conf.links[0].input.profile);
	TEST_ASSERT_EQUAL_HEX8(1, conf.links[0].input.channel_index);
	TEST_ASSERT_EQUAL_HEX16(0x1234, conf.links[0].input.owner_vid);
	TEST_ASSERT_EQUAL_HEX16(0x2345, conf.links[0].input.owner_pid);
	TEST_ASSERT_EQUAL_HEX16(0x3457, conf.links[0].input.owner_snum);
	TEST_ASSERT_EQUAL_HEX16(0x5679, conf.links[0].input.owner_type);
	TEST_ASSERT_EQUAL_HEX16(0x678A, conf.links[0].input.owner_profile);
	
	TEST_ASSERT_EQUAL_HEX16(0x5678, conf.links[0].output.type);
	TEST_ASSERT_EQUAL_HEX16(0x6789, conf.links[0].output.profile);
	TEST_ASSERT_EQUAL_HEX8(1, conf.links[0].output.channel_index);
	TEST_ASSERT_EQUAL_HEX16(0x1234, conf.links[0].output.owner_vid);
	TEST_ASSERT_EQUAL_HEX16(0x2345, conf.links[0].output.owner_pid);
	TEST_ASSERT_EQUAL_HEX16(0x3456, conf.links[0].output.owner_snum);
	TEST_ASSERT_EQUAL_HEX16(0x5678, conf.links[0].output.owner_type);
	TEST_ASSERT_EQUAL_HEX16(0x6789, conf.links[0].output.owner_profile);
	
	TEST_ASSERT_EQUAL_HEX16(0x5678, conf.links[1].input.type);
	TEST_ASSERT_EQUAL_HEX16(0x6789, conf.links[1].input.profile);
	TEST_ASSERT_EQUAL_HEX8(2, conf.links[1].input.channel_index);
	TEST_ASSERT_EQUAL_HEX16(0x1234, conf.links[1].input.owner_vid);
	TEST_ASSERT_EQUAL_HEX16(0x2345, conf.links[1].input.owner_pid);
	TEST_ASSERT_EQUAL_HEX16(0x3456, conf.links[1].input.owner_snum);
	TEST_ASSERT_EQUAL_HEX16(0x5678, conf.links[1].input.owner_type);
	TEST_ASSERT_EQUAL_HEX16(0x6789, conf.links[1].input.owner_profile);
	
	TEST_ASSERT_EQUAL_HEX16(0x5678, conf.links[1].output.type);
	TEST_ASSERT_EQUAL_HEX16(0x6789, conf.links[1].output.profile);
	TEST_ASSERT_EQUAL_HEX8(2, conf.links[1].output.channel_index);
	TEST_ASSERT_EQUAL_HEX16(0x1234, conf.links[1].output.owner_vid);
	TEST_ASSERT_EQUAL_HEX16(0x2345, conf.links[1].output.owner_pid);
	TEST_ASSERT_EQUAL_HEX16(0x3457, conf.links[1].output.owner_snum);
	TEST_ASSERT_EQUAL_HEX16(0x5679, conf.links[1].output.owner_type);
	TEST_ASSERT_EQUAL_HEX16(0x678A, conf.links[1].output.owner_profile);
	
	
	// uint8_t written[];
}

void test_saveLoadConfiguration(){
	write_system_info_response(2,3,2,3);
	write_device_info_response(0x56, 0x78, 0x89);
	write_device_info_response(0x57, 0x79, 0x8A);
	write_input_channel_info_response(0x56, 2, 0x5678, 0x6789);
	write_input_channel_info_response(0x57, 1, 0x5678, 0x6789);
	write_input_channel_info_response(0x57, 3, 0x5678, 0x6789);
	write_output_channel_info_response(0x56, 1, 0x5678, 0x6789);
	write_output_channel_info_response(0x57, 2, 0x5678, 0x6789);
	write_channel_link_info_response(0x56, 1, 0x57, 1, 1);
	write_channel_link_info_response(0x57, 2, 0x56, 2, 1);
	write_channel_link_info_response(0x56, 1, 0x57, 3, 1);
	
	lseek(conn.read_fd, 0, SEEK_SET);
	
	pdcp_system_info system_info;
	low_level_init(&system_info);
	
	system_info.channel_links[0].link_status=LINK_STATUS_ACTIVE;
	system_info.channel_links[1].link_status=LINK_STATUS_ACTIVE;
	system_info.channel_links[2].link_status=LINK_STATUS_INACTIVE;
	
	system_info.channel_links[0].link_status=LINK_STATUS_ACTIVE;
	system_info.channel_links[1].link_status=LINK_STATUS_ACTIVE;

	
	pdcp_configuration* conf = load_configuration();
	TEST_ASSERT_NULL(conf);
	save_configuration(&system_info);
	conf = load_configuration();
	TEST_ASSERT_NOT_NULL(conf);
	
	TEST_ASSERT_EQUAL(2, conf->n_links);
	 
	TEST_ASSERT_EQUAL_HEX16(0x5678, conf->links[0].input.type);
	TEST_ASSERT_EQUAL_HEX16(0x6789, conf->links[0].input.profile);
	TEST_ASSERT_EQUAL_HEX8(1, conf->links[0].input.channel_index);
	TEST_ASSERT_EQUAL_HEX16(0x1234, conf->links[0].input.owner_vid);
	TEST_ASSERT_EQUAL_HEX16(0x2345, conf->links[0].input.owner_pid);
	TEST_ASSERT_EQUAL_HEX16(0x3457, conf->links[0].input.owner_snum);
	TEST_ASSERT_EQUAL_HEX16(0x5679, conf->links[0].input.owner_type);
	TEST_ASSERT_EQUAL_HEX16(0x678A, conf->links[0].input.owner_profile);
	
	TEST_ASSERT_EQUAL_HEX16(0x5678, conf->links[0].output.type);
	TEST_ASSERT_EQUAL_HEX16(0x6789, conf->links[0].output.profile);
	TEST_ASSERT_EQUAL_HEX8(1, conf->links[0].output.channel_index);
	TEST_ASSERT_EQUAL_HEX16(0x1234, conf->links[0].output.owner_vid);
	TEST_ASSERT_EQUAL_HEX16(0x2345, conf->links[0].output.owner_pid);
	TEST_ASSERT_EQUAL_HEX16(0x3456, conf->links[0].output.owner_snum);
	TEST_ASSERT_EQUAL_HEX16(0x5678, conf->links[0].output.owner_type);
	TEST_ASSERT_EQUAL_HEX16(0x6789, conf->links[0].output.owner_profile);
	
	TEST_ASSERT_EQUAL_HEX16(0x5678, conf->links[1].input.type);
	TEST_ASSERT_EQUAL_HEX16(0x6789, conf->links[1].input.profile);
	TEST_ASSERT_EQUAL_HEX8(2, conf->links[1].input.channel_index);
	TEST_ASSERT_EQUAL_HEX16(0x1234, conf->links[1].input.owner_vid);
	TEST_ASSERT_EQUAL_HEX16(0x2345, conf->links[1].input.owner_pid);
	TEST_ASSERT_EQUAL_HEX16(0x3456, conf->links[1].input.owner_snum);
	TEST_ASSERT_EQUAL_HEX16(0x5678, conf->links[1].input.owner_type);
	TEST_ASSERT_EQUAL_HEX16(0x6789, conf->links[1].input.owner_profile);
	
	TEST_ASSERT_EQUAL_HEX16(0x5678, conf->links[1].output.type);
	TEST_ASSERT_EQUAL_HEX16(0x6789, conf->links[1].output.profile);
	TEST_ASSERT_EQUAL_HEX8(2, conf->links[1].output.channel_index);
	TEST_ASSERT_EQUAL_HEX16(0x1234, conf->links[1].output.owner_vid);
	TEST_ASSERT_EQUAL_HEX16(0x2345, conf->links[1].output.owner_pid);
	TEST_ASSERT_EQUAL_HEX16(0x3457, conf->links[1].output.owner_snum);
	TEST_ASSERT_EQUAL_HEX16(0x5679, conf->links[1].output.owner_type);
	TEST_ASSERT_EQUAL_HEX16(0x678A, conf->links[1].output.owner_profile);
}


void test_linksEqual(){
	pdcp_device dev1 = {
		.vendor_id = 1234,
		.product_id = 3456,
		.serial_num = 5678
	};
	
	pdcp_device dev2 = {
		.vendor_id = 4321,
		.product_id = 6543,
		.serial_num = 8765
	};

	pdcp_input_channel ichan = {
		.channel_index = 56,
		.owner = &dev1
	};
	pdcp_output_channel ochan = {
		.channel_index = 65,
		.owner = &dev2
	};

	pdcp_channel_link_configuration link_conf;
	
	link_conf.input.channel_index = 56;
	link_conf.input.owner_vid = 1234;
	link_conf.input.owner_pid = 3456;
	link_conf.input.owner_snum = 5678;
	link_conf.output.channel_index = 65;
	link_conf.output.owner_vid = 4321;
	link_conf.output.owner_pid = 6543;
	link_conf.output.owner_snum = 8765;
	
	pdcp_channel_link link_info = {
		.input_channel = &ichan,
		.output_channel = &ochan,
		.link_status = LINK_STATUS_ACTIVE
	};
	
	TEST_ASSERT(conf_links_equal(&link_conf, &link_info));
	
	link_conf.input.channel_index = 57;
	
	TEST_ASSERT_FALSE(conf_links_equal(&link_conf, &link_info));
	
	link_conf.input.channel_index = 56;
	link_conf.output.channel_index = 67;
	
	TEST_ASSERT_FALSE(conf_links_equal(&link_conf, &link_info));
	
	link_conf.output.channel_index = 65;
	link_conf.input.owner_vid = 1235;
	
	TEST_ASSERT_FALSE(conf_links_equal(&link_conf, &link_info));
	
	link_conf.input.owner_vid = 1234;
	link_conf.input.owner_pid = 3457;
	
	TEST_ASSERT_FALSE(conf_links_equal(&link_conf, &link_info));
	
	link_conf.input.owner_pid = 3456;
	link_conf.input.owner_snum = 5679;
	
	TEST_ASSERT_FALSE(conf_links_equal(&link_conf, &link_info));
	
	link_conf.input.owner_snum = 5678;
	link_conf.output.owner_vid = 4320;
	
	TEST_ASSERT_FALSE(conf_links_equal(&link_conf, &link_info));
	
	link_conf.output.owner_vid = 4321;
	link_conf.output.owner_pid = 6542;
	
	TEST_ASSERT_FALSE(conf_links_equal(&link_conf, &link_info));
	
	link_conf.output.owner_pid = 6543;
	link_conf.output.owner_snum = 8764;
	
	TEST_ASSERT_FALSE(conf_links_equal(&link_conf, &link_info));
}
