#include <errno.h>
#include <fcntl.h>
#include <unistd.h>

#include "pdcp_profiles.h"
#include "pdcp_low_level.h"
#include "pdcp_driver.h"
#include "pdcp_test_helper.h"
#include "unity.h"


uart_conn conn;


void setUp(){
	conn.read_fd = open(READ_PATH, O_RDWR | O_NOCTTY | O_SYNC | O_TRUNC);
	if(conn.read_fd < 0){
		char error_msg[25];
		sprintf(error_msg, "open failed: %d", errno);
	}
	conn.write_fd = open(WRITE_PATH, O_RDWR | O_NOCTTY | O_SYNC | O_TRUNC);
	if(conn.read_fd < 0){
		char error_msg[25];
		sprintf(error_msg, "open failed: %d", errno);
		TEST_FAIL_MESSAGE(error_msg);
	}
	test_helper_init(&conn);
}

void tearDown(){
	close(conn.read_fd);
}


void set_up_standard_test_system(){
	write_system_info_response(3,5,2,5);
	write_device_info_response(0x56, 0, 0);
	write_device_info_response(0x57, 0, 0);
	write_device_info_response(0x58, 0, 0);
	write_input_channel_info_response(0x56, 2, 0x78, 0x89);
	write_input_channel_info_response(0x57, 1, 0x78, 0x89);
	write_input_channel_info_response(0x58, 1, 0x78, 0x89);
	write_input_channel_info_response(0x58, 2, 0x78, 0x89);
	write_input_channel_info_response(0x58, 3, 0x78, 0x89);
	write_output_channel_info_response(0x56, 1, 0x78, 0x89);
	write_output_channel_info_response(0x57, 2, 0x78, 0x89);
	write_channel_link_info_response(0x56, 1, 0x57, 1, 1);
	write_channel_link_info_response(0x56, 1, 0x58, 1, 1);
	write_channel_link_info_response(0x57, 2, 0x56, 2, 1);
	write_channel_link_info_response(0x57, 2, 0x58, 2, 1);
	write_channel_link_info_response(0x59, 1, 0x58, 3, 1);
	
	lseek(conn.read_fd, 0, SEEK_SET);
	
	pdcp_system_info system_info;
	low_level_init(&system_info);
}





void find_replacements();
void enforce_sole_access();
int ochan_is_free(pdcp_output_channel* channel);
uint8_t determine_similarity_ichan(pdcp_input_channel* channel1, pdcp_input_channel* channel2);
uint8_t determine_similarity_ochan(pdcp_output_channel* channel1, pdcp_output_channel* channel2);

void auto_configure();
int input_channel_comparator(const void* elem1, const void* elem2);
int decide_channel_configure_order(pdcp_input_channel** out);


void test_isTerminalChannel(){
	pdcp_input_channel channel = {
		.type = 0b0000000100000000
	};
	TEST_ASSERT(is_terminal_channel(&channel))
	
	channel.type = ~channel.type;
	
	TEST_ASSERT_FALSE(is_terminal_channel(&channel))
	
}


//void test_

