//*********************************************************************
// 2011 Institute of Biomedical Engineering, UNB.
//
// FileName:        CAN1.h
//
// REVISION HISTORY:
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Author		Date      	Comments on this revision                                                                                                                                
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Y. Losier   11/08/12    MEC'11 PDCP Worshop
// A. Wilson            
//
// ADDITIONAL NOTES:
//
//*********************************************************************

#ifndef __CAN1_H__
#define __CAN1_H__

#include "p33fxxxx.h"
#include "Main.h"

typedef struct
{
   unsigned char priority;
   unsigned char mode;
   unsigned char nodeId;
   unsigned char length;

   union
   {
      unsigned char data8[8];
      unsigned int data16[4];
   };
}CANMESSAGE;


typedef struct
{
   unsigned int length;

   union
   {
      unsigned int UI16[4];
      unsigned char UI8[8];
   };
}CANdata;

extern CANdata canDataBuffer; // Define Outgoing Message Variable using CANdata Message Structure

#ifndef BITRATE_CAN1
   #define BITRATE_CAN1 1000000
#endif


// CAN Baud Rate Configuration
#define FC_CAN1      16000000
#define NTQ_CAN1       8        // 8 Time Quanta in a Bit Time
#define BRP_VAL_CAN1   ((FC_CAN1/(2*NTQ_CAN1*BITRATE_CAN1))-1)



// CAN Message Buffer Configuration
#define  ECAN1_MSG_BUF_LENGTH 	4
typedef unsigned int ECAN1MSGBUF [ECAN1_MSG_BUF_LENGTH][8];


#define CAN1_BINDING_MASK  0x7FF //  Setting the mask such that the device only listens to specific messages from the bus arbitrator
#define CAN1_STANDARD_MASK 0x000 //  Setting the mask such that the device listens to all messages on the CAN bus



//---------------------------------------------------------------
// Variable Declarations
//---------------------------------------------------------------
extern ECAN1MSGBUF  ecan1msgBuf __attribute__((space(dma),aligned(ECAN1_MSG_BUF_LENGTH*16)));

extern unsigned int CAN1Filter;
extern CANMESSAGE incomingMessage;

//-- End of Variable Declarations -------------------------------



//---------------------------------------------------------------
// Function Declarations
//---------------------------------------------------------------
extern void ecan1WriteRxAcptFilter(int n, long identifier, unsigned int exide,unsigned int bufPnt,unsigned int maskSel);
extern void ecan1WriteRxAcptMask(int m, long identifierMask, unsigned int mide,unsigned int exide);

extern void ecan1WriteTxMsgBufId(unsigned int buf, long txIdentifier, unsigned int ide, unsigned int remoteTransmit);
extern void ecan1WriteTxMsgBufData(unsigned int buf, unsigned int dataLength, unsigned int data1, unsigned int data2, unsigned int data3, unsigned int data4);

extern void CAN1_putId(unsigned int buf, long txIdentifier);
extern void CAN1_putBufData(unsigned int buf, CANdata* can1Data);
extern void CAN1_sendData(long address, CANdata* pCan1Data);

extern void __attribute__((interrupt, no_auto_psv))_C1Interrupt(void);
extern void __attribute__((interrupt, no_auto_psv))_DMA0Interrupt(void);
extern void __attribute__((interrupt, no_auto_psv))_DMA2Interrupt(void);

extern void CAN1PinConfig(void);
extern void CAN1ModuleConfig(unsigned filter);
extern void ecan1init(long canFilter, long canMask);
extern void dma0init(void);
extern void dma2init(void);

extern unsigned char MessageAvailable(void);
extern void ReadMessage(void);

extern unsigned char CanTxBusy(void);

//-- End of Function Declarations -------------------------------




#endif
