//*********************************************************************
// 2011 Institute of Biomedical Engineering, UNB.
//
// FileName:        PDCP.h
//
// REVISION HISTORY:
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Author		Date      	Comments on this revision                                                                                                                                
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Y. Losier   11/08/12    MEC'11 PDCP Worshop
// A. Wilson            
//
// ADDITIONAL NOTES:
//
//*********************************************************************

#ifndef __PDCP_H__
#define __PDCP_H__ 

#include "Main.h"
#include "CAN1.h"
#include "PDCP_parameters.h"
#include <stdint.h>
#include <stdio.h>


#define TEST_NUM                    4
#define DEVICE_NUM                  1


#define DUTY_CYCLE_PERIOD           15


extern unsigned char functionCode;
extern unsigned char responseNodeId;
extern unsigned int  responseVID;
extern unsigned int  responsePID;
extern unsigned int  responseSN;

extern unsigned char parameterId;
extern unsigned char channelIndex;

extern unsigned int messageSID;

extern pdcp_parameters* parameters_ptr;

//------------------------------------------------------------------------
// 
// UNB Prosthetic Device Communication Protocol Function Code Definitions
// 
//------------------------------------------------------------------------

#define PDCP_COMMAND_UNKNOWN                        0x00  // N/A.. YL

#define PDCP_COMMAND_BIND_REQUEST                   0x01
#define PDCP_COMMAND_GET_DEVICE_INFO                0x02  // Deprecated.. YL
#define PDCP_COMMAND_GET_DEVICE_PARAMETER           0x03
#define PDCP_COMMAND_SET_DEVICE_PARAMETER           0x04
#define PDCP_COMMAND_INDIRECT_GET_DEVICE_PARAMETER  0x05  // Deprecated.. YL
#define PDCP_COMMAND_INDIRECT_SET_DEVICE_PARAMETER  0x06  // Deprecated.. YL
#define PDCP_COMMAND_SET_NODE_ID                    0x07  // Deprecated.. YL
#define PDCP_COMMAND_SUSPEND_DEVICE                 0x08
#define PDCP_COMMAND_RELEASE_DEVICE                 0x09
#define PDCP_COMMAND_NODE_BEACON                    0x0A
#define PDCP_COMMAND_RESET_DEVICE                   0x0B
#define PDCP_COMMAND_CONFIG_GET_BULK_DATA_TRANSFER  0x0C
#define PDCP_COMMAND_CONFIG_SET_BULK_DATA_TRANSFER  0x0D
#define PDCP_COMMAND_BULK_DATA_TRANSFER             0x0E
#define PDCP_COMMAND_UPDATE_DATA_CHANNEL_LINK       0x0F

#define PDCP_RESPONSE_UNKNOWN                       0x80  // N/A.. YL

#define PDCP_RESPONSE_BIND_REQUEST                  0x81
#define PDCP_RESPONSE_GET_DEVICE_INFO               0x82  // Deprecated.. YL
#define PDCP_RESPONSE_GET_DEVICE_PARAMETER          0x83
#define PDCP_RESPONSE_SET_DEVICE_PARAMETER          0x84
#define PDCP_RESPONSE_INDIRECT_GET_DEVICE_PARAMETER 0x85  // Deprecated.. YL
#define PDCP_RESPONSE_INDIRECT_SET_DEVICE_PARAMETER 0x86  // Deprecated.. YL
#define PDCP_RESPONSE_SET_NODE_ID                   0x87  // Deprecated.. YL
#define PDCP_RESPONSE_SUSPEND_DEVICE                0x88
#define PDCP_RESPONSE_RELEASE_DEVICE                0x89
#define PDCP_RESPONSE_NODE_BEACON                   0x8A  // N/A.. YL
#define PDCP_RESPONSE_RESET_DEVICE                  0x8B
#define PDCP_RESPONSE_CONFIG_GET_BULK_DATA_TRANSFER 0x8C
#define PDCP_RESPONSE_CONFIG_SET_BULK_DATA_TRANSFER 0x8D
#define PDCP_RESPONSE_BULK_DATA_TRANSFER            0x8E
#define PDCP_RESPONSE_UPDATE_DATA_CHANNEL_LINK      0x8F

//------------------------------------------------------------------------




//------------------------------------------------------------------------
// 
// UNB Prosthetic Device Communication Protocol Various Definitions
// 
//------------------------------------------------------------------------

#define PDCP_RESPONSE_CODE_FAILURE    0x00
#define PDCP_RESPONSE_CODE_SUCCESSFUL 0x01

#define PDCP_PRIORITY_HIGH   0
#define PDCP_PRIORITY_NORMAL 1
#define PDCP_PRIORITY_LOW    2
#define PDCP_PRIORITY_BIND   3

#define PDCP_STANDARD_MESSAGE_MODE 0
#define PDCP_MASTER_MESSAGE_MODE   1

#define DEVICE_CHANNEL_INDEX 0

//------------------------------------------------------------------------



#define PDCP_TYPE_4_BIT                 0
#define PDCP_TYPE_8_BIT                 0b0100000000000000
#define PDCP_TYPE_12_BIT                0b1000000000000000
#define PDCP_TYPE_16_BIT                0b1100000000000000
#define PDCP_TYPE_INT                   0
#define PDCP_TYPE_UINT                  0b0001000000000000
#define PDCP_TYPE_FLOAT                 0b0010000000000000
#define PDCP_TYPE_BOOL                  0b0011000000000000
#define PDCP_TYPE_POSITION_MODE         0
#define PDCP_TYPE_VELOCITY_MODE         0b0000010000000000
#define PDCP_TYPE_ACCELERATION_MODE     0b0000100000000000
#define PDCP_TYPE_FORCE_MODE            0b0000110000000000
#define PDCP_TYPE_SOLE_ACCESS           0b0000001000000000
#define PDCP_TYPE_TERMINAL_CHANNEL      0b0000000100000000



#define BEACON_INTERVAL                                               100
#define UPDATE_DATA_CHANNEL_LINK_DELAY                                1000



typedef struct parameter_data PARAMETER_DATA;

struct parameter_data
{
   union
   {
      unsigned char data8[4];
      unsigned int data16[2];
   };
   
   unsigned char length;
};

extern PARAMETER_DATA parameterData;


typedef struct parameter_collection PARAMETER_COLLECTION;

struct parameter_collection
{
    PARAMETER_DATA* parameters;
    size_t n_parameters;
};


typedef struct system_parameters SYSTEM_PARAMETERS;

struct system_parameters{
    PARAMETER_COLLECTION device;
    PARAMETER_COLLECTION* channels;
    size_t n_channels;
};



//------------------------------------------------------------------------

extern CANMESSAGE * PDCP_GET_DEVICE_PARAMETER(CANMESSAGE * availableMessage, unsigned char nodeId, unsigned char parameterId, unsigned char channelIndex);
extern CANMESSAGE * PDCP_SET_DEVICE_PARAMETER(CANMESSAGE * availableMessage, unsigned char nodeId, unsigned char parameterId, unsigned char channelIndex, PARAMETER_DATA parameterData);
extern CANMESSAGE * PDCP_RESET_DEVICE(CANMESSAGE * availableMessage, unsigned char nodeId);


extern unsigned int createSID(unsigned char priority, unsigned char mode, unsigned char nodeId);

extern void SendBindRequestMessage(void);
extern void SendBeaconMessage(void);
extern void SendUpdateDataChannelLinkMessage(unsigned char channelIndex);
extern void SendDataSample(unsigned char DataChannelNodeId, unsigned int sampleValue);

extern void ProcessMessage(void);




#endif



