//*********************************************************************
// 2011 Institute of Biomedical Engineering, UNB.
//
// FileName:        CAN1.c
//
// REVISION HISTORY:
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Author		Date      	Comments on this revision                                                                                                                                
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Y. Losier   11/08/12    MEC'11 PDCP Worshop
// A. Wilson            
//
// ADDITIONAL NOTES:
//
//*********************************************************************

#include "p33fxxxx.h"
#include "CAN1.h"



//---------------------------------------------------------------
// Variable Declarations
//---------------------------------------------------------------
ECAN1MSGBUF ecan1msgBuf __attribute__((space(dma),aligned(ECAN1_MSG_BUF_LENGTH*16))); // Define ECAN Message Buffers in DMA memory space

unsigned int CAN1Filter;
CANMESSAGE incomingMessage;
CANdata canDataBuffer; // Define Outgoing Message Variable using CANdata Message Structure

//-- End of Variable Declarations -------------------------------



//*******************************************************************************
//
//    Function:     dma0init
//    Description:  DMA Initialization for ECAN1 Transmission
//
//*******************************************************************************
void dma0init(void)
{
	 DMACS0=0;
    DMA0CON=0x2020;
	 DMA0PAD=0x0442;	/* ECAN 1 (C1TXD) */
 	 DMA0CNT=0x0007;
	 DMA0REQ=0x0046;	/* ECAN 1 Transmit */
	 DMA0STA=  __builtin_dmaoffset(ecan1msgBuf);	
	 DMA0CONbits.CHEN=1;
}
//*******************************************************************************



//*******************************************************************************
//
//    Function:     dma2init
//    Description:  DMA Initialization for ECAN1 Reception
//
//*******************************************************************************
void dma2init(void)
{
	 DMACS0=0;
     DMA2CON=0x0020;
	 DMA2PAD=0x0440;	/* ECAN 1 (C1RXD) */
 	 DMA2CNT=0x0007;
	 DMA2REQ=0x0022;	/* ECAN 1 Receive */
	 DMA2STA= __builtin_dmaoffset(ecan1msgBuf);	
	 DMA2CONbits.CHEN=1;
}
//*******************************************************************************



//*******************************************************************************
//
//    Function:     ecan1ClkInit
//    Description:  Initialize ECAN1 Clock Parameters
//
//*******************************************************************************
void ecan1ClkInit(void)
{
	/* Synchronization Jump Width set to 1 TQ */
	C1CFG1bits.SJW = 0x0;
	/* Baud Rate Prescaler */
	C1CFG1bits.BRP = 0;

	/* Phase Segment 1 time is 3 TQ */
	C1CFG2bits.SEG1PH=0x2;
	/* Phase Segment 2 time is set to be programmable */
	C1CFG2bits.SEG2PHTS = 0x1;
	/* Phase Segment 2 time is 2 TQ */
	C1CFG2bits.SEG2PH = 0x1;
	/* Propagation Segment time is 2 TQ */
	C1CFG2bits.PRSEG = 0x1;
	/* Bus line is sampled three times at the sample point */
	C1CFG2bits.SAM = 0x1;
}
//*******************************************************************************



//*******************************************************************************
//
//    Function:     ecan1init
//    Description:  Initialize ECAN1 Module
//
//*******************************************************************************
void ecan1init(long canFilter, long canMask)
{

/* Request Configuration Mode */
	C1CTRL1bits.REQOP=4;
	while(C1CTRL1bits.OPMODE!=4);

	ecan1ClkInit();	


	C1FCTRLbits.DMABS=0b000;		/* 4 CAN Message Buffers in DMA RAM */

	ecan1WriteRxAcptFilter(1,canFilter,0,1,1);  // Standard
	ecan1WriteRxAcptMask(1,canMask,0,0);  // Standard
	

/* Enter Normal Mode */
	C1CTRL1bits.REQOP=0;
	while(C1CTRL1bits.OPMODE!=0);


/* ECAN transmit/receive message control */

	C1RXFUL1=C1RXFUL2=C1RXOVF1=C1RXOVF2=0x0000;
	C1TR01CONbits.TXEN0=1;			/* ECAN1, Buffer 0 is a Transmit Buffer */
	C1TR01CONbits.TXEN1=0;			/* ECAN1, Buffer 1 is a Receive Buffer */
	C1TR01CONbits.TX0PRI=0b11; 		/* Message Buffer 0 Priority Level */
	C1TR01CONbits.TX1PRI=0b11; 		/* Message Buffer 1 Priority Level */
}
//*******************************************************************************



//*******************************************************************************
//
//    Function:     ecan1WriteRxAcptFilter
//    Description:  Initialize ECAN1 RX Filter
//
//*******************************************************************************
void ecan1WriteRxAcptFilter(int n, long identifier, unsigned int exide, unsigned int bufPnt,unsigned int maskSel)
{
	unsigned long sid10_0=0, eid15_0=0, eid17_16=0;
	unsigned int *sidRegAddr,*bufPntRegAddr,*maskSelRegAddr, *fltEnRegAddr;

	C1CTRL1bits.WIN=1;

	// Obtain the Address of CiRXFnSID, CiBUFPNTn, CiFMSKSELn and CiFEN register for a given filter number "n"
	sidRegAddr = (unsigned int *)(&C1RXF0SID + (n << 1));
	bufPntRegAddr = (unsigned int *)(&C1BUFPNT1 + (n >> 2));
	maskSelRegAddr = (unsigned int *)(&C1FMSKSEL1 + (n >> 3));
	fltEnRegAddr = (unsigned int *)(&C1FEN1);


	// Bit-filed manupulation to write to Filter identifier register
	if(exide==1) { 	// Filter Extended Identifier
		eid15_0 = (identifier & 0xFFFF);
		eid17_16= (identifier>>16) & 0x3;
		sid10_0 = (identifier>>18) & 0x7FF;

		*sidRegAddr=(((sid10_0)<<5) + 0x8) + eid17_16;	// Write to CiRXFnSID Register
	    *(sidRegAddr+1)= eid15_0;					// Write to CiRXFnEID Register

	}else{			// Filter Standard Identifier
		sid10_0 = (identifier & 0x7FF);			
		*sidRegAddr=(sid10_0)<<5;					// Write to CiRXFnSID Register
		*(sidRegAddr+1)=0;							// Write to CiRXFnEID Register
	}


   *bufPntRegAddr = (*bufPntRegAddr) & (0xFFFF - (0xF << (4 *(n & 3)))); // clear nibble
   *bufPntRegAddr = ((bufPnt << (4 *(n & 3))) | (*bufPntRegAddr));       // Write to C1BUFPNTn Register

   *maskSelRegAddr = (*maskSelRegAddr) & (0xFFFF - (0x3 << ((n & 7) * 2))); // clear 2 bits
   *maskSelRegAddr = ((maskSel << (2 * (n & 7))) | (*maskSelRegAddr));      // Write to C1FMSKSELn Register

   *fltEnRegAddr = ((0x1 << n) | (*fltEnRegAddr)); // Write to C1FEN1 Register

   C1CTRL1bits.WIN=0;
}
//*******************************************************************************



//*******************************************************************************
//
//    Function:     ecan1WriteRxAcptMask
//    Description:  Initialize ECAN1 RX Mask
//
//*******************************************************************************
void ecan1WriteRxAcptMask(int m, long identifier, unsigned int mide, unsigned int exide)
{
	unsigned long sid10_0=0, eid15_0=0, eid17_16=0;
	unsigned int *maskRegAddr;

	C1CTRL1bits.WIN=1;

	// Obtain the Address of CiRXMmSID register for given Mask number "m"
	maskRegAddr = (unsigned int *)(&C1RXM0SID + (m << 1));

	// Bit-filed manupulation to write to Filter Mask register
	if(exide==1) 
	{ 	// Filter Extended Identifier
		eid15_0 = (identifier & 0xFFFF);
		eid17_16= (identifier>>16) & 0x3;
		sid10_0 = (identifier>>18) & 0x7FF;

		if(mide==1)
			*maskRegAddr=((sid10_0)<<5) + 0x0008 + eid17_16;	// Write to CiRXMnSID Register
		else
			*maskRegAddr=((sid10_0)<<5) + eid17_16;	// Write to CiRXMnSID Register
	    *(maskRegAddr+1)= eid15_0;					// Write to CiRXMnEID Register

	}
	else
	{			// Filter Standard Identifier
		sid10_0 = (identifier & 0x7FF);			
		if(mide==1)
			*maskRegAddr=((sid10_0)<<5) + 0x0008;					// Write to CiRXMnSID Register
		else
			*maskRegAddr=(sid10_0)<<5;					// Write to CiRXMnSID Register	
		
		*(maskRegAddr+1)=0;							// Write to CiRXMnEID Register
	}

	C1CTRL1bits.WIN=0;	
}
//*******************************************************************************



//*******************************************************************************
//
//    Function:     ecan1WriteTxMsgBufId
//    Description:  Store ECAN1 Transmission Identifier
//
//*******************************************************************************
void ecan1WriteTxMsgBufId(unsigned int buf, long txIdentifier, unsigned int ide, unsigned int remoteTransmit)
{
	unsigned long word0=0, word1=0, word2=0;
	unsigned long sid10_0=0, eid5_0=0, eid17_6=0;

		
	if(ide)
	{
		eid5_0  = (txIdentifier & 0x3F);
		eid17_6 = (txIdentifier>>6) & 0xFFF;
		sid10_0 = (txIdentifier>>18) & 0x7FF;
		word1 = eid17_6;
	}
	else
	{
		sid10_0 = (txIdentifier & 0x7FF);
	}

		
	if(remoteTransmit==1) 	// Transmit Remote Frame
	{
		word0 = ((sid10_0 << 2) | ide | 0x2);
		word2 = ((eid5_0 << 10)| 0x0200);
 	}  
	else
	{
		word0 = ((sid10_0 << 2) | ide);
		word2 = (eid5_0 << 10);
	}
		
				
	// Obtain the Address of Transmit Buffer in DMA RAM for a given Transmit Buffer number
	if(ide)
	{
		ecan1msgBuf[buf][0] = (word0 | 0x0002);
	}
	else
	{
		ecan1msgBuf[buf][0] = word0;
	}
	
	ecan1msgBuf[buf][1] = word1;
	ecan1msgBuf[buf][2] = word2;
}
//*******************************************************************************



//*******************************************************************************
//
//    Function:     ecan1WriteTxMsgBufData
//    Description:  Store ECAN1 Transmission Identifier
//
//*******************************************************************************
void ecan1WriteTxMsgBufData(unsigned int buf, unsigned int dataLength, unsigned int data1, unsigned int data2, unsigned int data3, unsigned int data4)
{
	ecan1msgBuf[buf][2] = ((ecan1msgBuf[buf][2] & 0xFFF0) + dataLength) ;
	
	ecan1msgBuf[buf][3] = data1;
	ecan1msgBuf[buf][4] = data2;
	ecan1msgBuf[buf][5] = data3;
	ecan1msgBuf[buf][6] = data4;
}
//*******************************************************************************



//*******************************************************************************
//
//    Function:     CAN1_putId
//    Description:  Compact Function to Store ECAN1 Transmission Identifier
//
//*******************************************************************************
void CAN1_putId(unsigned int buf, long txIdentifier)
{
	unsigned int ide = 0;
	unsigned int remoteTransmit = 0;
	
	unsigned long word0=0, word1=0, word2=0;
	unsigned long sid10_0=0, eid5_0=0, eid17_6=0;
		
	if(ide)
	{
		eid5_0  = (txIdentifier & 0x3F);
		eid17_6 = (txIdentifier>>6) & 0xFFF;
		sid10_0 = (txIdentifier>>18) & 0x7FF;
		word1 = eid17_6;
	}
	else
	{
		sid10_0 = (txIdentifier & 0x7FF);
	}
	
	
	if(remoteTransmit==1)  	// Transmit Remote Frame
	{
		word0 = ((sid10_0 << 2) | ide | 0x2);
		word2 = ((eid5_0 << 10)| 0x0200);
	}	
	else
	{
		word0 = ((sid10_0 << 2) | ide);
		word2 = (eid5_0 << 10);
	}
				
	// Obtain the Address of Transmit Buffer in DMA RAM for a given Transmit Buffer number
	
	if(ide)
	{
		ecan1msgBuf[buf][0] = (word0 | 0x0002);
	}
	else
	{
		ecan1msgBuf[buf][0] = word0;
	}

	ecan1msgBuf[buf][1] = word1;
	ecan1msgBuf[buf][2] = word2;
}
//*******************************************************************************



//*******************************************************************************
//
//    Function:     CAN1_putBufData
//    Description:  Compact Function to Store ECAN1 Transmission Data
//
//*******************************************************************************
void CAN1_putBufData(unsigned int buf, CANdata* can1Data)
{
	ecan1msgBuf[buf][2] = ((ecan1msgBuf[buf][2] & 0xFFF0) + can1Data->length) ;
	
	ecan1msgBuf[buf][3] = can1Data->UI16[0];
	ecan1msgBuf[buf][4] = can1Data->UI16[1];
	ecan1msgBuf[buf][5] = can1Data->UI16[2];
	ecan1msgBuf[buf][6] = can1Data->UI16[3];
}
//*******************************************************************************



//*******************************************************************************
//
//    Function:     CAN1_sendData
//    Description:  Transmit CAN Message on ECAN1 Bus
//
//*******************************************************************************
void CAN1_sendData(long address, CANdata* pCan1Data)
{
   while (C1TR01CONbits.TXREQ0) {}
   CAN1_putId(0,address);
   CAN1_putBufData(0,pCan1Data);
   C1TR01CONbits.TXREQ0=1;
}
//*******************************************************************************



//*******************************************************************************
//
//    Function:     CAN1 ISR
//    Description:  Handles interrupts caused incoming or outgoing CAN activity
//                  (not doing anything other than setting a flag)
//
//*******************************************************************************
void __attribute__((interrupt, no_auto_psv))_C1Interrupt(void)
{
   // check to see if the interrupt is caused by receive or transmit
   if(C1INTFbits.RBIF)
   {
      C1INTFbits.RBIF = 0;  // clear flag
   }
   else if(C1INTFbits.TBIF)
   {
      C1INTFbits.TBIF = 0;  // clear flag
   }

   IFS2bits.C1IF = 0;  // clear interrupt flag
}
//*******************************************************************************



//*******************************************************************************
//                                                                             
//    Function:       CAN1PinConfig
//    Description:    Configures CAN1 Pins
//                                                                             
//*******************************************************************************
void CAN1PinConfig(void)
{
   // CAN1 RX
   _TRISB6 = 1;  // input digital i/o

   // CAN1 TX
   _TRISB7 = 0;  // output digital i/o
   
   
   //*************************************************************
	// Unlock Registers:
	//
	// Clear the bit 6 of OSCCONL to unlock Pin Re-map
	//
	//*************************************************************
	__builtin_write_OSCCONL(OSCCON & 0xbf);
	//*************************************************************

	_C1RXR = 6;  // CAN1 RX mapped onto RP6
	_RP7R  = 16; // CAN1 TX mapped onto RP7

	//************************************************************
	// Lock Registers:
	//
	// Set the bit 6 of OSCCONL to lock Pin Re-map
	//
	//************************************************************
	__builtin_write_OSCCONL(OSCCON | 0x40);
	//*************************************************************
   
   
   // CAN1 AMCU EN
   _TRISB5 = 0;  // output digital i/o
   _LATB5  = 1;

   // CAN1 AMCU STANDBY
   _TRISB4 = 0;  // output digital i/o
   _LATB4  = 0;
}
//*******************************************************************************



//*******************************************************************************
//                                                                             
//    Function:       CAN1ModuleConfig
//    Description:    Configures CAN1 Module
//                                                                             
//*******************************************************************************
void CAN1ModuleConfig(unsigned filter)
{
   CAN1Filter = filter;
   
   ecan1init(CAN1Filter,CAN1_BINDING_MASK);
	dma0init();	
	dma2init();
	IEC2bits.C1IE = 1;
	C1INTEbits.TBIE = 1;	
	C1INTEbits.RBIE = 1;
}
//*******************************************************************************



//*******************************************************************************
//                                                                             
//    Function:       DMA0 ISR
//    Description:    Handles DMA interrupt caused by outgoing CAN1 activity
//                       (currently not doing anything other than clearing flag)
//                                                                             
//*******************************************************************************
void __attribute__((interrupt, no_auto_psv)) _DMA0Interrupt(void)
{
   IFS0bits.DMA0IF = 0;          // Clear the DMA0 Interrupt Flag;
}
//*******************************************************************************



//*******************************************************************************
//                                                                             
//    Function:       DMA2 ISR
//    Description:    Handles DMA interrupt caused by incoming CAN1 activity
//                       (currently not doing anything other than clearing flag)
//                                                                             
//*******************************************************************************
void __attribute__((interrupt, no_auto_psv)) _DMA2Interrupt(void)
{
   IFS1bits.DMA2IF = 0;          // Clear the DMA2 Interrupt Flag;
}
//*******************************************************************************



//*******************************************************************************
//                                                                             
//    Function:       MessageAvailable
//    Description:    Returns whether there's a message available of not
//                                                                             
//*******************************************************************************
unsigned char MessageAvailable(void)
{
   if(C1RXFUL1bits.RXFUL1==1)
   {
      return 1;
   }
   else
   {
      return 0;
   }         
}
//*******************************************************************************



//*******************************************************************************
//                                                                             
//    Function:       ReadMessage
//    Description:    Reads current incoming message
//                                                                             
//*******************************************************************************
void ReadMessage(void)
{
   unsigned int ide=0;
   unsigned int srr=0;
   unsigned long id=0;

   // read word 0 to see the message type
   ide=ecan1msgBuf[1][0] & 0x0001;
   srr=ecan1msgBuf[1][0] & 0x0002;

   // check to see what type of message it is
   if(ide==0)  // message is standard identifier
   {
      incomingMessage.nodeId=(unsigned char)(ecan1msgBuf[1][0] >> 2);
      incomingMessage.mode=(unsigned char)((ecan1msgBuf[1][0] >> 10) & 0x01);
      incomingMessage.priority=(unsigned char)(ecan1msgBuf[1][0] >> 11);
   }
   else  // mesage is extended identifier
   {
      id=ecan1msgBuf[1][0] & 0x1FFC;
      incomingMessage.nodeId=id << 16;
      id=ecan1msgBuf[1][1] & 0x0FFF;
      incomingMessage.nodeId=incomingMessage.nodeId+(id << 6);
      id=(ecan1msgBuf[1][2] & 0xFC00) >> 10;
      incomingMessage.nodeId=incomingMessage.nodeId+id;
   }

   // check to see what type of message it is
   if(srr==1)  // RTR message
   {

   }
   else  // normal message
   {
      incomingMessage.data16[0]=ecan1msgBuf[1][3];
      incomingMessage.data16[1]=ecan1msgBuf[1][4];
      incomingMessage.data16[2]=ecan1msgBuf[1][5];
      incomingMessage.data16[3]=ecan1msgBuf[1][6];

      incomingMessage.length=(unsigned char)(ecan1msgBuf[1][2] & 0x000F);

   }
   
   C1RXFUL1bits.RXFUL1=0;
}
//*******************************************************************************



//*******************************************************************************
//                                                                             
//    Function:       CanTxBusy
//    Description:    Checks to see if device is currently sending a CAN message
//                                                                             
//*******************************************************************************
unsigned char CanTxBusy(void)
{
   if (C1TR01CONbits.TXREQ0)
   {
      return 1;   
   }
   else
   {
      return 0;
   }
}
//*******************************************************************************



