//*********************************************************************
// 2011 Institute of Biomedical Engineering, UNB.
//
// FileName:        Main.c
//
// REVISION HISTORY:
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Author		Date      	Comments on this revision                                                                                                                                
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Y. Losier            11/08/12            MEC'11 PDCP Worshop
// A. Wilson            
//
// . Rnningstad       Spring 2013         Master thesis
// ADDITIONAL NOTES:
//
//*********************************************************************

#include "p33Fxxxx.h"
#include "Main.h"
#include "Timer.h"
#include "CAN1.h"
#include "PDCP.h"
#include "I2C.h"
#include "ADC1.h"
#include "DEE Emulation 16-bit.h"
#include "PWM.h"


//---------------------------------------------------------------
// Bind Sequence Defines and Variables Declarations
//---------------------------------------------------------------
#define BINDREQUEST_TIMEOUT_INTERVAL (100)
#define MAX_NUM_OF_TIMEOUTS (50)

unsigned int initialDelayValue;
unsigned char deviceBound = 0;
unsigned char numOfTimeouts = 0;
unsigned long responseTimeout;
//-- End of Bind Sequence Defines and Variables Declarations ----


//---------------------------------------------------------------
// Variable Declarations 
//---------------------------------------------------------------
unsigned char testMode = 0;
unsigned char timer1Flag = 0;
unsigned char currentFlag = 0;
unsigned long timer1FlagCounter = 25;
unsigned long nextBeaconTime = 0;
unsigned long startUpdateDataChannelLinkRequestTime;
unsigned char updateDataChannelLinkOnPOR = 0;

/*
unsigned char currentDeviceNodeId;
unsigned char dataChannel1NodeId = 0;
unsigned char dataChannel1TransferEnableFlag = 0;
unsigned int  dataChannel1SourceVID = 0xFFFF;
unsigned int  dataChannel1SourcePID = 0xFFFF;
unsigned int  dataChannel1SourceSN  = 0xFFFF;
unsigned int  dataChannel1SourceCI  = 0xFFFF;
unsigned char dataChannel1SourceDataNodeId = 0xFF;
unsigned char dataChannel1SourceDeviceNodeId = 0xFF;

unsigned int  dataChannel1SourceType  = 0x0000;
unsigned int  dataChannel1SourceProfile  = 0x0000;
*/

//---------------------------------------------------------------


//---------------------------------------------------------------
// Function Declarations 
//---------------------------------------------------------------
void UpdateBarGraph(unsigned int value);
void UpdateBarGraphRaw(unsigned int barFlags);

void dutyCycle11();
void dutyCycle12();
void dutyCycle13();
void dutyCycle21();
void dutyCycle22();
void dutyCycle23();
void dutyCycle31();
void dutyCycle32();
void dutyCycle33();
void dutyCycle41();
void dutyCycle42();
void dutyCycle43();
void dutyCycle51();
void dutyCycle52();
void dutyCycle53();
void dutyCycle61();
void dutyCycle62();
void dutyCycle63();
void dutyCycle71();
void dutyCycle72();
void dutyCycle73();
//-- End of Function Declarations -------------------------------


//---------------------------------------------------------------
// Oscillator Configuration Registers Assignments
//---------------------------------------------------------------
// External Oscillator (32MHz)
_FOSCSEL(FNOSC_PRI);                             // Primary Oscillator
_FOSC(FCKSM_CSECMD & OSCIOFNC_ON  & POSCMD_EC);  // Clock Switching enabled and Fail Safe Clock Monitor is disabled
                                                 // OSC2 Pin Function: OSC2 is a Digital I/O
                                                 // Primary Oscillator Mode: EC Oscillator (32MHz - 16MIPS)

_FWDT(FWDTEN_OFF);                               // Watchdog Timer Enabled/disabled by user software
//-- End of Oscillator Configuration Registers Assignments ------


//---------------------------------------------------
// Instruction Clock Speed Definition
//---------------------------------------------------
#define FCY       16000000
//-- End of Instruction Clock Speed Definition ------



//---------------------------------------------------------------
// main () 
//---------------------------------------------------------------
int main(void)
{
    
   // --------------------------------------------------------------
   // Non-volatile memory initialization
   // --------------------------------------------------------------
   DataEEInit();
   dataEEFlags.val = 0;
   // --------------------------------------------------------------


   // --------------------------------------------------------------
   // Joystick Pushbutton Pin Configuration
   // --------------------------------------------------------------
   _TRISA1= 1;  // input i/o
   _PCFG1 = 1;  // digital pin
   // --------------------------------------------------------------


   // --------------------------------------------------------------
   // Retrieving saved parameters_ptr from non-volatile memory
   // --------------------------------------------------------------
   if (DataEERead(DEE_ADDR_EMULATION_FLAG)==0xFFFF) // Initial POR of current firmware version
   {
      parameters_ptr[0].device.node_id = parameters_ptr[0].device.snum & 0x0F;  // Choose arbitrary Node Id value for device

      DataEEWrite(1, DEE_ADDR_EMULATION_FLAG);  // Set Flag in n-v memory to indicate that Initial POR occured
      DataEEWrite(parameters_ptr[0].device.node_id, DEE_ADDR_DEVICE_NODE_ID);       // Write variable to n-v memory

      int i;
      for (i = 1; i <= parameters_ptr[0].device.n_data_channels; i++){
          if (parameters_ptr[i].transfer_type == TRANSFER_TYPE_INPUT){
              parameters_ptr[i].ichan.source_vid = 0xFFFF;              // Clear Source VID variable
              parameters_ptr[i].ichan.source_pid = 0xFFFF;              // Clear Source PID variable
              parameters_ptr[i].ichan.source_snum  = 0xFFFF;              // Clear Source SN variable
              parameters_ptr[i].ichan.source_channel_index  = 0xFF;                // Clear Source CI variable
              parameters_ptr[i].ichan.source_type = 0;
              parameters_ptr[i].ichan.source_profile = 0;

              DataEEWrite(parameters_ptr[i].ichan.source_vid,DEE_ADDR_SOURCE_VID + DEE_CHANNEL_OFFSET + ((i-1) * DEE_CHANNEL_SIZE));         // Write variable to n-v memory
              DataEEWrite(parameters_ptr[i].ichan.source_pid,DEE_ADDR_SOURCE_PID + DEE_CHANNEL_OFFSET + ((i-1) * DEE_CHANNEL_SIZE));         // Write variable to n-v memory
              DataEEWrite(parameters_ptr[i].ichan.source_snum,DEE_ADDR_SOURCE_SN + DEE_CHANNEL_OFFSET + ((i-1) * DEE_CHANNEL_SIZE));           // Write variable to n-v memory
              DataEEWrite(parameters_ptr[i].ichan.source_channel_index,DEE_ADDR_SOURCE_CHANNEL_INDEX + DEE_CHANNEL_OFFSET + ((i-1) * DEE_CHANNEL_SIZE));// Write variable to n-v memory
              DataEEWrite(parameters_ptr[i].ichan.source_type,DEE_ADDR_SOURCE_TYPE + DEE_CHANNEL_OFFSET + ((i-1) * DEE_CHANNEL_SIZE));
              DataEEWrite(parameters_ptr[i].ichan.source_profile,DEE_ADDR_SOURCE_PROFILE + DEE_CHANNEL_OFFSET + ((i-1) * DEE_CHANNEL_SIZE));
          }
      }
   }
   else  // Initial POR already occured previously
   {
       parameters_ptr[0].device.node_id                  = DataEERead(DEE_ADDR_DEVICE_NODE_ID);       // Retrieve last used device node Id from memory

       int i;
       for (i = 1; i <= parameters_ptr[0].device.n_data_channels; i++){
           if (parameters_ptr[i].transfer_type == TRANSFER_TYPE_INPUT){
               parameters_ptr[i].ichan.source_vid                = DataEERead(DEE_ADDR_SOURCE_VID + DEE_CHANNEL_OFFSET + ((i-1) * DEE_CHANNEL_SIZE));           // Retrieve last configured Source VID value from memory
               parameters_ptr[i].ichan.source_pid                = DataEERead(DEE_ADDR_SOURCE_PID + DEE_CHANNEL_OFFSET + ((i-1) * DEE_CHANNEL_SIZE));           // Retrieve last configured Source PID value from memory
               parameters_ptr[i].ichan.source_snum               = DataEERead(DEE_ADDR_SOURCE_SN + DEE_CHANNEL_OFFSET + ((i-1) * DEE_CHANNEL_SIZE));            // Retrieve last configured Source SN value from memory
               parameters_ptr[i].ichan.source_channel_index      = DataEERead(DEE_ADDR_SOURCE_CHANNEL_INDEX + DEE_CHANNEL_OFFSET + ((i-1) * DEE_CHANNEL_SIZE)); // Retrieve last configured Source CI value from memory
               parameters_ptr[i].ichan.source_type               = DataEERead(DEE_ADDR_SOURCE_TYPE + DEE_CHANNEL_OFFSET + ((i-1) * DEE_CHANNEL_SIZE));
               parameters_ptr[i].ichan.source_profile            = DataEERead(DEE_ADDR_SOURCE_PROFILE + DEE_CHANNEL_OFFSET + ((i-1) * DEE_CHANNEL_SIZE));
           }
       }
   }
   // --------------------------------------------------------------


   // --------------------------------------------------------------
   // Initialization Functions
   // --------------------------------------------------------------
   ClearInterruptFlags();
   Config_Microstick_LED1_Pin();

   PWM_Init();
   InitTimer1();
    
   CAN1PinConfig();
   CAN1ModuleConfig((0x300 + parameters_ptr[0].device.node_id));
    
   InitI2C();			
   InitAdc1();

   StartTimer1();
   // -- End of Initialization -------------------------------------




   //---------------------------------------------------------------
   // Bind Sequence
   //---------------------------------------------------------------

   UpdateBarGraph(0);
   //UpdateBarGraphRaw(0b0000001010101010);

   unsigned s_snum = parameters_ptr[2].ichan.source_snum;

   pdcp_parameters* chan2 = &parameters_ptr[2];

   initialDelayValue = (VENDOR_ID + PRODUCT_ID + parameters_ptr[0].device.snum) & 0x000F;  // Generate an initial delay value (between 0 and 15ms)

   while (timer1Counter<initialDelayValue) {}  // wait an initial delay before starting the binding process...

   SendBindRequestMessage();

   responseTimeout = timer1Counter + BINDREQUEST_TIMEOUT_INTERVAL;  // set value for timeout (if response isn't received)


   
   while (!deviceBound)
   {
      while ((!MessageAvailable() ) && (timer1Counter < responseTimeout)) {}  // wait for either a response or a timeout
   	
      if (MessageAvailable())
      {

         ReadMessage();

         functionCode   = incomingMessage.data8[0];
         responseNodeId = incomingMessage.data8[1];
         responseVID    = incomingMessage.data16[1];
         responsePID    = incomingMessage.data16[2];
         responseSN     = incomingMessage.data16[3];

         if ((functionCode == PDCP_RESPONSE_BIND_REQUEST) && (responseVID == VENDOR_ID) && (responsePID == PRODUCT_ID) && (responseSN == parameters_ptr[0].device.snum)) // if the message is for this device AND it is a bind request response...
         {
            if (responseNodeId == parameters_ptr[0].device.node_id)  // was the nodeId accepted?
            {
               deviceBound = 1;
               LED1_ON();//UpdateBarGraph(100);

               ecan1init(CAN1Filter,CAN1_STANDARD_MASK);
               
               DataEEWrite(parameters_ptr[0].device.node_id,DEE_ADDR_DEVICE_NODE_ID);
               
               nextBeaconTime = timer1Counter + BEACON_INTERVAL;
               startUpdateDataChannelLinkRequestTime = timer1Counter + UPDATE_DATA_CHANNEL_LINK_DELAY;
            }
            else  // Bus Arbitrator suggested using a different nodeId...
            {
               parameters_ptr[0].device.node_id = responseNodeId;
               CAN1Filter = (0x300 + parameters_ptr[0].device.node_id);
               ecan1init(CAN1Filter,CAN1_BINDING_MASK);
               SendBindRequestMessage();
               responseTimeout = timer1Counter + BINDREQUEST_TIMEOUT_INTERVAL;  // reset next timeout value...
            }
         }
         else // message is either for some other device AND/OR isn't a bind request response...
         {
            // don't care because the device isn't bounded yet... no need to process it.
         }

      }
      else  // Timeout has occured
      {
         
         numOfTimeouts++;
         UpdateBarGraph(numOfTimeouts);
         
         if (numOfTimeouts < MAX_NUM_OF_TIMEOUTS)
         {
            SendBindRequestMessage();
            responseTimeout = timer1Counter + BINDREQUEST_TIMEOUT_INTERVAL;  // reset next timeout value...
         }
         else  // If device exceeded max number of binding attempts...
         {
            while (1)
            {
               Sleep();  // PowerDown Device
            }
         }
      }
   }
   // -- End of Bind Sequence --------------------------------------


   //---------------------------------------------------------------
   // Endless While Loop 
   //---------------------------------------------------------------
    //unsigned PWM_value = 0;
   while(1)
	{
   	currentFlag = timer1Flag;  // Storing timer1 status prior to entering loop

        

      //------------------------------------------------------------------------
      // Beacon Transmission: Send a beacon message at a fixed interval (500ms)
      //------------------------------------------------------------------------
      if (nextBeaconTime < timer1Counter)
      {
         nextBeaconTime = timer1Counter + BEACON_INTERVAL;
         SendBeaconMessage();

        // PWM_value = PWM_value < 2048 ? PWM_value+100 : 0;

        // PWM_set(PWM_value);
      }
      //------------------------------------------------------------------------


      //------------------------------------------------------------------------
      // CAN Message Handling: Read and process any available incoming message
      //------------------------------------------------------------------------ 
      if (MessageAvailable())
      {
         ReadMessage();
         ProcessMessage();  
      }
      //------------------------------------------------------------------------

      if (currentFlag){
        #if TEST_NUM == 1
            #if DEVICE_NUM == 1
                    dutyCycle11();
            #elif DEVICE_NUM == 2
                    dutyCycle12();
            #elif DEVICE_NUM == 3
                    dutyCycle13();
            #endif
        #elif TEST_NUM == 2
            #if DEVICE_NUM == 1
                    dutyCycle21();
            #elif DEVICE_NUM == 2
                    dutyCycle22();
            #elif DEVICE_NUM == 3
                    dutyCycle23();
            #endif
        #elif TEST_NUM == 3
            #if DEVICE_NUM == 1
                    dutyCycle31();
            #elif DEVICE_NUM == 2
                    dutyCycle32();
            #elif DEVICE_NUM == 3
                    dutyCycle33();
            #endif
        #elif TEST_NUM == 4
            #if DEVICE_NUM == 1
                    dutyCycle41();
            #elif DEVICE_NUM == 2
                    dutyCycle42();
            #elif DEVICE_NUM == 3
                    dutyCycle43();
            #endif
        #elif TEST_NUM == 5
            #if DEVICE_NUM == 1
                    dutyCycle51();
            #elif DEVICE_NUM == 2
                    dutyCycle52();
            #elif DEVICE_NUM == 3
                    dutyCycle53();
            #endif
        #elif TEST_NUM == 6
            #if DEVICE_NUM == 1
                    dutyCycle61();
            #elif DEVICE_NUM == 2
                    dutyCycle62();
            #elif DEVICE_NUM == 3
                    dutyCycle63();
            #endif
        #elif TEST_NUM == 7
            #if DEVICE_NUM == 1
                    dutyCycle71();
            #elif DEVICE_NUM == 2
                    dutyCycle72();
            #elif DEVICE_NUM == 3
                    dutyCycle73();
            #endif
        #elif TEST_NUM == 8
            #if DEVICE_NUM == 1
                    dutyCycle81();
            #elif DEVICE_NUM == 2
                    dutyCycle82();
            #elif DEVICE_NUM == 3
                    dutyCycle83();
            #endif
        #elif TEST_NUM == 9
            #if DEVICE_NUM == 1
                    dutyCycle91();
            #elif DEVICE_NUM == 2
                    dutyCycle92();
            #elif DEVICE_NUM == 3
                    dutyCycle93();
            #endif
        #endif
      }
      //------------------------------------------------------------------------
      // Initial Update Data Channel Link Query: 2sec after being bound, device
      //                                           sends out a UDCL command
      //------------------------------------------------------------------------ 
      // Wait until device has been bound for a given interval (UPDATE_DATA_CHANNEL_LINK_DELAY == 2000ms)
      if ((timer1Counter > startUpdateDataChannelLinkRequestTime) && (updateDataChannelLinkOnPOR == 0))
      {
          int i;
          for (i = 0; i <= parameters_ptr[0].device.n_data_channels; i++){
             // if input data channel is not enabled, request an update from the bus arbitrator..
             if (parameters_ptr[i].transfer_type == TRANSFER_TYPE_INPUT && parameters_ptr[i].chan.transfer_enable_flag==0)
             {
                SendUpdateDataChannelLinkMessage(i); // Channel Index = i
             }
          }
         updateDataChannelLinkOnPOR = 1;  // Set flag to indicate that update messages have been sent to bus arbitrator
      }
      //------------------------------------------------------------------------


      //---------------------------------------------------------------------------------------
      // Timer Housekeeping: Verifying if timer1 has been updated since last loop iteration
      //---------------------------------------------------------------------------------------
      if ((currentFlag == timer1Flag) && (timer1Flag == 1))
      {
   	   timer1FlagCounter = timer1Counter + DUTY_CYCLE_PERIOD;
   	   timer1Flag = 0;
   	}
      //---------------------------------------------------------------------------------------



	} //end of while(1)
} //end of main()
//---------------------------------------------------------------




//*******************************************************************************
//
//    Function:     UpdateBarGraph
//    Description:  Updates Bar Graph using a value between 0 and 100
//
//*******************************************************************************
void UpdateBarGraph(unsigned int value)
{
   unsigned char tens;
   unsigned char round;
   unsigned char numOfBars;
   unsigned char i;
   unsigned int  barFlags = 0;

   unsigned char LSB;
   unsigned char MSB;

   if (value > 100)
   {
      value = 100;
   }
   
   tens = value / 10;
   round = (value / 5) % 2;

   numOfBars = tens + round;   
   
   for (i=0;i<numOfBars;i++)
   {
      barFlags = barFlags  | (1 << i);
   }

   LSB = ((barFlags >> 0) & 0xFF);
   MSB = ((barFlags >> 8) & 0xFF);
   
   LDByteWriteI2C(writeControlByte, 0x14, LSB);
   LDByteWriteI2C(writeControlByte, 0x15, MSB);
}
//*******************************************************************************


void UpdateBarGraphRaw(unsigned int barFlags){
   unsigned char LSB = ((barFlags >> 0) & 0xFF);
   unsigned char MSB = ((barFlags >> 8) & 0xFF);

   LDByteWriteI2C(writeControlByte, 0x14, LSB);
   LDByteWriteI2C(writeControlByte, 0x15, MSB);
}

void sendYandXandButton(int yChannelIndex, int xChannelIndex, int buttonChannelIndex){
     int sent_something = 0;
     //UpdateBarGraph((ain3Buff[0] >> 4)/2.56);
    unsigned char y_value = ain3Buff[0] >> 4;
    unsigned char x_value = ain4Buff[0] >> 4;

     if (parameters_ptr[yChannelIndex].chan.transfer_enable_flag == 1)  // If data streaming is enabled
     {
        SendDataSample(parameters_ptr[yChannelIndex].ochan.node_id, y_value);  // Y-axis
        sent_something = 1;
     }

     if (parameters_ptr[xChannelIndex].chan.transfer_enable_flag == 1)  // If data streaming is enabled
     {
        SendDataSample(parameters_ptr[xChannelIndex].ochan.node_id, x_value);  // X-axis
        UpdateBarGraph(x_value/2.56);
        sent_something = 1;
     }
     else{
        UpdateBarGraph(y_value/2.56);
     }

     if (parameters_ptr[buttonChannelIndex].chan.transfer_enable_flag == 1)  // If data streaming is enabled
     {
        SendDataSample(parameters_ptr[buttonChannelIndex].ochan.node_id, _RA1);  // button
        sent_something = 1;
     }

    if (sent_something){
        LED1_TOGGLE();
    }
     _SAMP = 0;  // Start next sample conversions

}

int sendY(int channelIndex){
    if (parameters_ptr[channelIndex].chan.transfer_enable_flag == 1)  // If data streaming is enabled
     {
        SendDataSample(parameters_ptr[channelIndex].ochan.node_id, ain3Buff[0] >> 4);  // Y-axis
        _SAMP = 0;  // Start next sample conversions
        return 1;
     }
     _SAMP = 0;  // Start next sample conversions
    return 0;
}

int sendX(int channelIndex){
    if (parameters_ptr[channelIndex].chan.transfer_enable_flag == 1)  // If data streaming is enabled
     {
        SendDataSample(parameters_ptr[channelIndex].ochan.node_id, ain4Buff[0] >> 4);  // X-axis
        _SAMP = 0;  // Start next sample conversions
        return 1;
     }
     _SAMP = 0;  // Start next sample conversions
     return 0;
}

int sendButton(int channelIndex){
    if (parameters_ptr[channelIndex].chan.transfer_enable_flag == 1)  // If data streaming is enabled
     {
        SendDataSample(parameters_ptr[channelIndex].ochan.node_id, _RA1);  // button
        _SAMP = 0;  // Start next sample conversions
        return 1;
     }
     _SAMP = 0;  // Start next sample conversions
    return 0;
}



void updateLEDBar(int channelIndex){
    if(parameters_ptr[channelIndex].ichan.has_received_value){
        UpdateBarGraph((parameters_ptr[channelIndex].ichan.received_value)/2.56);
        parameters_ptr[channelIndex].ichan.has_received_value = 0;
        //LED1_TOGGLE();
    }
}

void updateLEDPWM(int channelIndex){
    if(parameters_ptr[channelIndex].ichan.has_received_value){
        PWM_set((parameters_ptr[channelIndex].ichan.received_value)*8);
        parameters_ptr[channelIndex].ichan.has_received_value = 0;
        //LED1_TOGGLE();
    }
}

void updateLEDBinary(int channelIndex){
    if(parameters_ptr[channelIndex].ichan.has_received_value){
        PWM_set(parameters_ptr[channelIndex].ichan.received_value * 2047);
        parameters_ptr[channelIndex].ichan.has_received_value = 0;
    }
}



void dutyCycle11(){
    if(sendX(1))
        LED1_TOGGLE();
}

void dutyCycle12(){
    updateLEDBar(1);
}

void dutyCycle13(){
    updateLEDBinary(1);
    updateLEDBar(2);
}


void dutyCycle21(){
    sendY(1);
}

void dutyCycle22(){
    if (parameters_ptr[2].ichan.has_received_value){
        unsigned char arrayValue, pwmValue, binaryValue;
        unsigned char inValue = parameters_ptr[2].ichan.received_value;
        if (parameters_ptr[1].chan.transfer_enable_flag){ // receiving button values
            if (parameters_ptr[1].ichan.has_received_value){
                if (parameters_ptr[1].ichan.received_value){ // button pressed
                    pwmValue = inValue;
                    arrayValue = 0;
                } else { // button not pressed
                    pwmValue = 0;
                    arrayValue = inValue;
                }
            } else {
                return; // wait for button value
            }
        } else { // not receiving button values
            pwmValue = 0;
            arrayValue = inValue;
        }
        binaryValue = (inValue > 127);
        if (parameters_ptr[3].chan.transfer_enable_flag == 1){  // If data streaming is enabled
            SendDataSample(parameters_ptr[3].ochan.node_id, arrayValue);
        }
        if (parameters_ptr[4].chan.transfer_enable_flag == 1){  // If data streaming is enabled
            SendDataSample(parameters_ptr[4].ochan.node_id, pwmValue);
        }
        if (parameters_ptr[5].chan.transfer_enable_flag == 1){  // If data streaming is enabled
            SendDataSample(parameters_ptr[5].ochan.node_id, binaryValue);
        }
    }
}

void dutyCycle23(){
    updateLEDBar(1);
    updateLEDBinary(2);
}


void dutyCycle31(){
    dutyCycle21();
}

void dutyCycle32(){
    dutyCycle22();
}

void dutyCycle33(){
    dutyCycle23();
}


void dutyCycle41(){
    sendX(1);
}

void dutyCycle42(){
    sendButton(1);
    sendY(2);
}

void dutyCycle43(){
    updateLEDBinary(1);
    updateLEDBar(2);
}


void dutyCycle51(){
    dutyCycle21();
}

void dutyCycle52(){
    dutyCycle22();
}

void dutyCycle53(){
    dutyCycle23();
}


void dutyCycle61(){

}

void dutyCycle62(){

}

void dutyCycle63(){

}


void dutyCycle71(){
   sendYandXandButton(1, 2, 3);
}

void dutyCycle72(){
    updateLEDBar(1);
    updateLEDBinary(2);
}

void dutyCycle73(){
    updateLEDPWM(1);
}

