//*********************************************************************
// 2011 Institute of Biomedical Engineering, UNB.
//
// FileName:        PDCP.c
//
// REVISION HISTORY:
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Author               Date            Comments on this revision                                                                                                                                
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// Y. Losier   11/08/12    MEC'11 PDCP Worshop
// A. Wilson            
//
// ADDITIONAL NOTES:
//
//*********************************************************************

#include "PDCP.h"
#include "CAN1.h"
#include "DEE Emulation 16-bit.h"

PARAMETER_DATA parameterData;


unsigned char functionCode;
unsigned char responseNodeId;
unsigned int responseVID;
unsigned int responsePID;
unsigned int responseSN;

unsigned char parameterId;
unsigned char channelIndex;

unsigned int messageSID;







#if TEST_NUM == 1
#if DEVICE_NUM == 1

#define N_PARAMETERS 2;

pdcp_parameters parameters[] = {
    {
     .device.firmware_version = FIRMWARE_VERSION,
     .device.hardware_version = HARDWARE_VERSION,
     .device.vid = VENDOR_ID,
     .device.pid = PRODUCT_ID,
     .transfer_type = TRANSFER_TYPE_DEVICE_DUMMY,

     .device.n_data_channels = 1,
     .device.snum = 1,
     .device.type = 0,
     .device.profile = 0},
    {
     .chan.type = PDCP_TYPE_8_BIT | PDCP_TYPE_UINT,
     .chan.profile = 0x2110,
     .transfer_type = TRANSFER_TYPE_OUTPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0}
};


#elif DEVICE_NUM == 2

#define N_PARAMETERS 2;

pdcp_parameters parameters[] = {
    {
     .device.firmware_version = FIRMWARE_VERSION,
     .device.hardware_version = HARDWARE_VERSION,
     .device.vid = VENDOR_ID,
     .device.pid = PRODUCT_ID,
     .transfer_type = TRANSFER_TYPE_DEVICE_DUMMY,

     .device.n_data_channels = 1,
     .device.snum = 2,
     .device.type = 0,
     .device.profile = 0},
    {
     .chan.type = PDCP_TYPE_8_BIT | PDCP_TYPE_UINT | PDCP_TYPE_POSITION_MODE | PDCP_TYPE_SOLE_ACCESS | PDCP_TYPE_TERMINAL_CHANNEL,
     .chan.profile = 0x2110,
     .ichan.terminal_profile = 0x1110,
     .ichan.received_value = 0,
     .ichan.has_received_value = 0,
     .transfer_type = TRANSFER_TYPE_INPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0}
};


#elif DEVICE_NUM == 3

#define N_PARAMETERS 3;

pdcp_parameters parameters[] = {
    {
     .device.firmware_version = FIRMWARE_VERSION,
     .device.hardware_version = HARDWARE_VERSION,
     .device.vid = VENDOR_ID,
     .device.pid = PRODUCT_ID,
     .transfer_type = TRANSFER_TYPE_DEVICE_DUMMY,

     .device.n_data_channels = 2,
     .device.snum = 3,
     .device.type = 0,
     .device.profile = 0},
    {
     .chan.type = PDCP_TYPE_8_BIT | PDCP_TYPE_BOOL | PDCP_TYPE_POSITION_MODE | PDCP_TYPE_SOLE_ACCESS | PDCP_TYPE_TERMINAL_CHANNEL,
     .chan.profile = 0x2200,
     .ichan.terminal_profile = 0x1220,
     .ichan.received_value = 0,
     .ichan.has_received_value = 0,
     .transfer_type = TRANSFER_TYPE_INPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0},
    {
     .chan.type = PDCP_TYPE_8_BIT | PDCP_TYPE_UINT | PDCP_TYPE_POSITION_MODE | PDCP_TYPE_SOLE_ACCESS | PDCP_TYPE_TERMINAL_CHANNEL,
     .chan.profile = 0x2120,
     .ichan.terminal_profile = 0x1110,
     .ichan.received_value = 0,
     .ichan.has_received_value = 0,
     .transfer_type = TRANSFER_TYPE_INPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0}
};

#endif

#elif TEST_NUM == 2 || TEST_NUM == 3 || TEST_NUM == 5
#if DEVICE_NUM == 1

#define N_PARAMETERS 2;

pdcp_parameters parameters[] = {
    {
     .device.firmware_version = FIRMWARE_VERSION,
     .device.hardware_version = HARDWARE_VERSION,
     .device.vid = VENDOR_ID,
     .device.pid = PRODUCT_ID,
     .transfer_type = TRANSFER_TYPE_DEVICE_DUMMY,

     .device.n_data_channels = 1,
     .device.snum = 4,
     .device.type = 0,
     .device.profile = 0},
    {
     .chan.type = PDCP_TYPE_8_BIT | PDCP_TYPE_UINT | PDCP_TYPE_POSITION_MODE,
     .chan.profile = 0x2120,
     .transfer_type = TRANSFER_TYPE_OUTPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0}
};


#elif DEVICE_NUM == 2

#define N_PARAMETERS 6;
pdcp_parameters parameters[] = {
    {
     .device.firmware_version = FIRMWARE_VERSION,
     .device.hardware_version = HARDWARE_VERSION,
     .device.vid = VENDOR_ID,
     .device.pid = PRODUCT_ID,
     .transfer_type = TRANSFER_TYPE_DEVICE_DUMMY,

     .device.n_data_channels = 5,
     .device.snum = 5,
     .device.type = 0,
     .device.profile = 0},
    {
     .chan.type = PDCP_TYPE_8_BIT | PDCP_TYPE_BOOL | PDCP_TYPE_POSITION_MODE | PDCP_TYPE_SOLE_ACCESS,
     .chan.profile = 0x2200,
     .ichan.terminal_profile = 0,
     .ichan.received_value = 0,
     .ichan.has_received_value = 0,
     .transfer_type = TRANSFER_TYPE_INPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0},
    {
     .chan.type = PDCP_TYPE_8_BIT | PDCP_TYPE_UINT | PDCP_TYPE_POSITION_MODE | PDCP_TYPE_SOLE_ACCESS,
     .chan.profile = 0x2110,
     .ichan.terminal_profile = 0,
     .ichan.received_value = 0,
     .ichan.has_received_value = 0,
     .transfer_type = TRANSFER_TYPE_INPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0},
    {
     .chan.type = PDCP_TYPE_8_BIT | PDCP_TYPE_UINT | PDCP_TYPE_POSITION_MODE,
     .chan.profile = 0x1100,
     .transfer_type = TRANSFER_TYPE_OUTPUT,
     .chan.required_input_channels = 0b00100000,
     .chan.desired_input_channels = 0},
    {
     .chan.type = PDCP_TYPE_8_BIT | PDCP_TYPE_UINT | PDCP_TYPE_POSITION_MODE,
     .chan.profile = 0x1210,
     .transfer_type = TRANSFER_TYPE_OUTPUT,
     .chan.required_input_channels = 0b01000000,
     .chan.desired_input_channels = 0},
    {
     .chan.type = PDCP_TYPE_8_BIT | PDCP_TYPE_BOOL | PDCP_TYPE_POSITION_MODE,
     .chan.profile = 0x1220,
     .transfer_type = TRANSFER_TYPE_OUTPUT,
     .chan.required_input_channels = 0b00100000,
     .chan.desired_input_channels = 0}
};


#elif DEVICE_NUM == 3

#define N_PARAMETERS 3;

pdcp_parameters parameters[] = {
    {
     .device.firmware_version = FIRMWARE_VERSION,
     .device.hardware_version = HARDWARE_VERSION,
     .device.vid = VENDOR_ID,
     .device.pid = PRODUCT_ID,
     .transfer_type = TRANSFER_TYPE_DEVICE_DUMMY,

     .device.n_data_channels = 2,
     .device.snum = 6,
     .device.type = 0,
     .device.profile = 0},
    {
     .chan.type = PDCP_TYPE_8_BIT | PDCP_TYPE_UINT | PDCP_TYPE_POSITION_MODE | PDCP_TYPE_SOLE_ACCESS | PDCP_TYPE_TERMINAL_CHANNEL,
     .chan.profile = 0x1110,
     .ichan.terminal_profile = 0x1110,
     .ichan.received_value = 0,
     .ichan.has_received_value = 0,
     .transfer_type = TRANSFER_TYPE_INPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0},
    {
     .chan.type = PDCP_TYPE_8_BIT | PDCP_TYPE_BOOL | PDCP_TYPE_POSITION_MODE | PDCP_TYPE_SOLE_ACCESS | PDCP_TYPE_TERMINAL_CHANNEL,
     .chan.profile = 0x1220,
     .ichan.terminal_profile = 0x1220,
     .ichan.received_value = 0,
     .ichan.has_received_value = 0,
     .transfer_type = TRANSFER_TYPE_INPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0}
};

#endif

#elif TEST_NUM == 4
#if DEVICE_NUM == 1

#define N_PARAMETERS ;
pdcp_parameters parameters[] = {
    {
     .device.firmware_version = FIRMWARE_VERSION,
     .device.hardware_version = HARDWARE_VERSION,
     .device.vid = VENDOR_ID,
     .device.pid = PRODUCT_ID,
     .transfer_type = TRANSFER_TYPE_DEVICE_DUMMY,

     .device.n_data_channels = 1,
     .device.snum = 7,
     .device.type = 0,
     .device.profile = 0},
     {
     .chan.type = PDCP_TYPE_8_BIT | PDCP_TYPE_UINT | PDCP_TYPE_POSITION_MODE,
     .chan.profile = 0x2110,
     .transfer_type = TRANSFER_TYPE_OUTPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0}
};


#elif DEVICE_NUM == 2

#define N_PARAMETERS ;
pdcp_parameters parameters[] = {
    {
     .device.firmware_version = FIRMWARE_VERSION,
     .device.hardware_version = HARDWARE_VERSION,
     .device.vid = VENDOR_ID,
     .device.pid = PRODUCT_ID,
     .transfer_type = TRANSFER_TYPE_DEVICE_DUMMY,

     .device.n_data_channels = 2,
     .device.snum = 8,
     .device.type = 0,
     .device.profile = 0},
     {
     .chan.type = PDCP_TYPE_8_BIT | PDCP_TYPE_BOOL | PDCP_TYPE_POSITION_MODE,
     .chan.profile = 0x2200,
     .transfer_type = TRANSFER_TYPE_OUTPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0},
     {
     .chan.type = PDCP_TYPE_8_BIT | PDCP_TYPE_UINT | PDCP_TYPE_POSITION_MODE,
     .chan.profile = 0x2120,
     .transfer_type = TRANSFER_TYPE_OUTPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0}
};


#elif DEVICE_NUM == 3

#define N_PARAMETERS ;
pdcp_parameters parameters[] = {
    {
     .device.firmware_version = FIRMWARE_VERSION,
     .device.hardware_version = HARDWARE_VERSION,
     .device.vid = VENDOR_ID,
     .device.pid = PRODUCT_ID,
     .transfer_type = TRANSFER_TYPE_DEVICE_DUMMY,

     .device.n_data_channels = 2,
     .device.snum = 9,
     .device.type = 0,
     .device.profile = 0},
     {
     .chan.type = PDCP_TYPE_8_BIT | PDCP_TYPE_BOOL | PDCP_TYPE_POSITION_MODE | PDCP_TYPE_SOLE_ACCESS | PDCP_TYPE_TERMINAL_CHANNEL,
     .chan.profile = 0x2200,
     .ichan.terminal_profile = 0x2200,
     .ichan.received_value = 0,
     .ichan.has_received_value = 0,
     .transfer_type = TRANSFER_TYPE_INPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0},
     {
     .chan.type = PDCP_TYPE_8_BIT | PDCP_TYPE_UINT | PDCP_TYPE_POSITION_MODE | PDCP_TYPE_SOLE_ACCESS | PDCP_TYPE_TERMINAL_CHANNEL,
     .chan.profile = 0x2110,
     .ichan.terminal_profile = 0x2110,
     .ichan.received_value = 0,
     .ichan.has_received_value = 0,
     .transfer_type = TRANSFER_TYPE_INPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0}
};
#endif

#elif TEST_NUM == 7
#if DEVICE_NUM == 1

#define N_PARAMETERS 4;
pdcp_parameters parameters[] = {
    {
     .device.firmware_version = FIRMWARE_VERSION,
     .device.hardware_version = HARDWARE_VERSION,
     .device.vid = VENDOR_ID,
     .device.pid = PRODUCT_ID,
     .transfer_type = TRANSFER_TYPE_DEVICE_DUMMY,

     .device.n_data_channels = 3,
     .device.snum = 18,
     .device.type = 0,
     .device.profile = 0},
   {
         // joystick y
     .chan.type = 0b0101000000000000, // 8 bit unsigned int
     .chan.profile = 0x2120,
     .transfer_type = TRANSFER_TYPE_OUTPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0},
   {
         // joystick x
     .chan.type = 0b0101000000000000, // 8 bit unsigned int
     .chan.profile = 0x2110,
     .transfer_type = TRANSFER_TYPE_OUTPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0},
   {
         // button
     .chan.type = 0b0111000000000000, // 8 bit boolean
     .chan.profile = 0x2200,
     .transfer_type = TRANSFER_TYPE_OUTPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0}
};


#elif DEVICE_NUM == 2

#define N_PARAMETERS 3;
pdcp_parameters parameters[] = {
    {
     .device.firmware_version = FIRMWARE_VERSION,
     .device.hardware_version = HARDWARE_VERSION,
     .device.vid = VENDOR_ID,
     .device.pid = PRODUCT_ID,
     .transfer_type = TRANSFER_TYPE_DEVICE_DUMMY,

     .device.n_data_channels = 2,
     .device.snum = 19,
     .device.type = 0,
     .device.profile = 0},
   {
     .chan.type = 0b0101001100000000,
     .chan.profile = 0x2100,
     .ichan.terminal_profile = 0x1110,
     .ichan.received_value = 0,
     .ichan.has_received_value = 0,
     .transfer_type = TRANSFER_TYPE_INPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0},
   {
     .chan.type = 0b0101001100000000,
     .chan.profile = 0x2200,
     .ichan.terminal_profile = 0x1220,
     .ichan.received_value = 0,
     .ichan.has_received_value = 0,
     .transfer_type = TRANSFER_TYPE_INPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0}
};


#elif DEVICE_NUM == 3

#define N_PARAMETERS 3;
pdcp_parameters parameters[] = {
    {
     .device.firmware_version = FIRMWARE_VERSION,
     .device.hardware_version = HARDWARE_VERSION,
     .device.vid = VENDOR_ID,
     .device.pid = PRODUCT_ID,
     .transfer_type = TRANSFER_TYPE_DEVICE_DUMMY,

     .device.n_data_channels = 1,
     .device.snum = 20,
     .device.type = 0,
     .device.profile = 0},
   {
     .chan.type = 0b0101001100000000,
     .chan.profile = 0x2100,
     .ichan.terminal_profile = 0x1110,
     .ichan.received_value = 0,
     .ichan.has_received_value = 0,
     .transfer_type = TRANSFER_TYPE_INPUT,
     .chan.required_input_channels = 0,
     .chan.desired_input_channels = 0}
};
#endif

#endif


pdcp_parameters *parameters_ptr = &parameters[0];



CANMESSAGE *PDCP_GET_DEVICE_PARAMETER(CANMESSAGE * availableMessage, unsigned char nodeId, unsigned char parameterId, unsigned char channelIndex)
{
    availableMessage->priority = PDCP_PRIORITY_NORMAL;
    availableMessage->mode = PDCP_MASTER_MESSAGE_MODE;
    availableMessage->nodeId = nodeId;
    availableMessage->data8[0] = PDCP_COMMAND_GET_DEVICE_PARAMETER;
    availableMessage->data8[1] = parameterId;
    availableMessage->data8[2] = channelIndex;
    availableMessage->length = 3;

    return availableMessage;
}

CANMESSAGE *PDCP_SET_DEVICE_PARAMETER(CANMESSAGE * availableMessage, unsigned char nodeId, unsigned char parameterId, unsigned char channelIndex, PARAMETER_DATA parameterData)
{
    availableMessage->priority = PDCP_PRIORITY_NORMAL;
    availableMessage->mode = PDCP_MASTER_MESSAGE_MODE;
    availableMessage->nodeId = nodeId;
    availableMessage->data8[0] = PDCP_COMMAND_SET_DEVICE_PARAMETER;
    availableMessage->data8[1] = parameterId;
    availableMessage->data8[2] = channelIndex;
    availableMessage->data8[3] = parameterData.data8[0];
    availableMessage->data8[4] = parameterData.data8[1];
    availableMessage->data8[5] = parameterData.data8[2];
    availableMessage->data8[6] = parameterData.data8[3];

    availableMessage->length = (3 + parameterData.length);

    return availableMessage;
}

CANMESSAGE *PDCP_RESET_DEVICE(CANMESSAGE * availableMessage, unsigned char nodeId)
{
    availableMessage->priority = PDCP_PRIORITY_NORMAL;
    availableMessage->mode = PDCP_MASTER_MESSAGE_MODE;
    availableMessage->nodeId = nodeId;
    availableMessage->data8[0] = PDCP_COMMAND_RESET_DEVICE;
    availableMessage->length = 1;

    return availableMessage;
}



//*******************************************************************************
//                                                                             
//    Function:       createSID
//    Description:    Creates SID value based on priority, mode, and nodeId
//                     
//    Arguments:      priority, mode, nodeId
//                                                                             
//*******************************************************************************
unsigned int createSID(unsigned char priority, unsigned char mode, unsigned char nodeId)
{
    unsigned int SID = 0;
//   
//   SID=priority;
//   SID=SID<<1;
//   SID=SID|mode;
//   SID=SID<<8;
//   SID=SID|nodeId;
//
    SID = ((((unsigned int) priority) << 9) | (((unsigned int) mode) << 8)
	   | (unsigned int) nodeId);

    return SID;
}

//*******************************************************************************



//*******************************************************************************
//                                                                             
//    Function:       SendBindRequestMessage
//    Description:    Sends bind request message to bus arbitrator
//                                                                             
//*******************************************************************************
void SendBindRequestMessage(void)
{
    unsigned int canSID;

    canDataBuffer.length = 7;
    canDataBuffer.UI8[0] = PDCP_COMMAND_BIND_REQUEST;
    canDataBuffer.UI8[1] = (unsigned char) (VENDOR_ID);
    canDataBuffer.UI8[2] = (unsigned char) (VENDOR_ID >> 8);
    canDataBuffer.UI8[3] = (unsigned char) (PRODUCT_ID);
    canDataBuffer.UI8[4] = (unsigned char) (PRODUCT_ID >> 8);
    canDataBuffer.UI8[5] = (unsigned char) (parameters[0].device.snum);
    canDataBuffer.UI8[6] = (unsigned char) (parameters[0].device.snum >> 8);

    canSID = createSID(PDCP_PRIORITY_BIND, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

    CAN1_sendData(canSID, &canDataBuffer);
}

//*******************************************************************************



//*******************************************************************************
//                                                                             
//    Function:       SendBeaconMessage
//    Description:    Sends beacon message to notify its presence on the bus
//                                                                             
//*******************************************************************************
void SendBeaconMessage(void)
{
    unsigned int canSID;

    canDataBuffer.length = 1;
    canDataBuffer.UI8[0] = PDCP_COMMAND_NODE_BEACON;

    canSID = createSID(PDCP_PRIORITY_HIGH, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

    CAN1_sendData(canSID, &canDataBuffer);
}

//*******************************************************************************



//*******************************************************************************
//                                                                             
//    Function:       SendBeaconMessage
//    Description:    Sends beacon message to notify its presence on the bus
//                                                                             
//*******************************************************************************
void SendUpdateDataChannelLinkMessage(unsigned char channelIndex)
{
    unsigned int canSID;

    canDataBuffer.length = 2;
    canDataBuffer.UI8[0] = PDCP_COMMAND_UPDATE_DATA_CHANNEL_LINK;
    canDataBuffer.UI8[1] = channelIndex;
    canSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

    CAN1_sendData(canSID, &canDataBuffer);
}

//*******************************************************************************



//*******************************************************************************
//                                                                             
//    Function:       SendData
//    Description:    Sends data sample on bus
//                                                                             
//*******************************************************************************
void SendDataSample(unsigned char DataChannelNodeId, unsigned int sampleValue)
{
    unsigned int canSID;

    canDataBuffer.length = 1;
    canDataBuffer.UI8[0] = (unsigned char)sampleValue;
    canSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, DataChannelNodeId);

    CAN1_sendData(canSID, &canDataBuffer);
}

//*******************************************************************************





//*******************************************************************************
//                                                                             
//    Function:       ProcessMessage
//    Description:    Processes the command/response found within the given
//                      can1 message
//                                                                             
//*******************************************************************************
void ProcessMessage(void)
{
    // Check to make sure message is intended for this device...
    //if (((incomingMessage.nodeId != parameters[0].device.node_id)
//	 || (incomingMessage.mode != PDCP_MASTER_MESSAGE_MODE))) {
//	return;
  //  }


    if (incomingMessage.priority != PDCP_PRIORITY_BIND) {
	if (incomingMessage.mode != PDCP_MASTER_MESSAGE_MODE) {
	    int i;
	    for (i = 1; i <= parameters[0].device.n_data_channels; i++) {
		if (parameters[i].transfer_type == TRANSFER_TYPE_INPUT && incomingMessage.nodeId == parameters[i].ichan.source_channel_node_id) {

                    parameters[i].ichan.received_value = incomingMessage.data8[0];
                    parameters[i].ichan.has_received_value = 1;
		    return;
		}
	    }
	}
    }

    functionCode = incomingMessage.data8[0];

    // Check to see if message is intended for another device trying to bind itself and, if so, disregard message...
    if (functionCode == PDCP_RESPONSE_BIND_REQUEST || incomingMessage.nodeId != parameters[0].device.node_id) {
	return;
    }

    switch (functionCode) {
    case PDCP_COMMAND_GET_DEVICE_PARAMETER:
	parameterId = incomingMessage.data8[1];
	channelIndex = incomingMessage.data8[2];

	if (channelIndex == DEVICE_CHANNEL_INDEX)	// if channelIndex is zero, it's a device-specific parameter...
	{
	    switch (parameterId) {
	    case PARAMETER_ADDRESS_DEVICE_VID_PID:
		canDataBuffer.length = 8;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;
		canDataBuffer.UI8[4] = (unsigned char) (VENDOR_ID);
		canDataBuffer.UI8[5] = (unsigned char) (VENDOR_ID >> 8);
		canDataBuffer.UI8[6] = (unsigned char) (PRODUCT_ID);
		canDataBuffer.UI8[7] = (unsigned char) (PRODUCT_ID >> 8);

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    case PARAMETER_ADDRESS_DEVICE_SERIAL_NUMBER:
		canDataBuffer.length = 6;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;
		canDataBuffer.UI8[4] = (unsigned char) (parameters[0].device.snum);
		canDataBuffer.UI8[5] = (unsigned char) (parameters[0].device.snum >> 8);

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    case PARAMETER_ADDRESS_DEVICE_EAN13L:
		canDataBuffer.length = 4;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_FAILURE;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    case PARAMETER_ADDRESS_DEVICE_EAN13H:
		canDataBuffer.length = 4;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_FAILURE;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    case PARAMETER_ADDRESS_DEVICE_FW_HW_VER:
		canDataBuffer.length = 8;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;
		canDataBuffer.UI8[4] = (unsigned char) (FIRMWARE_VERSION);
		canDataBuffer.UI8[5] = (unsigned char) (FIRMWARE_VERSION >> 8);
		canDataBuffer.UI8[6] = (unsigned char) (HARDWARE_VERSION);
		canDataBuffer.UI8[7] = (unsigned char) (HARDWARE_VERSION >> 8);

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    case PARAMETER_ADDRESS_DEVICE_TYPE_PROFILE:
		canDataBuffer.length = 8;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;
		canDataBuffer.UI8[4] = (unsigned char) (parameters[0].device.type);
		canDataBuffer.UI8[5] = (unsigned char) (parameters[0].device.type >> 8);
		canDataBuffer.UI8[6] = (unsigned char) (parameters[0].device.profile);
		canDataBuffer.UI8[7] = (unsigned char) (parameters[0].device.profile >> 8);

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    case PARAMETER_ADDRESS_DEVICE_DESCRIPTOR:
		canDataBuffer.length = 4;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_FAILURE;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    case PARAMETER_ADDRESS_DEVICE_NODE_ID:
		canDataBuffer.length = 5;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;
		canDataBuffer.UI8[4] = parameters[0].device.node_id;

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    case PARAMETER_ADDRESS_DEVICE_NUM_OF_DATA_CHANNELS:
		canDataBuffer.length = 5;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;
		canDataBuffer.UI8[4] = (unsigned char) parameters[0].device.n_data_channels;

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    default:
		canDataBuffer.length = 4;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_FAILURE;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    }
	} else			//if (channelIndex >= 1)
	{
	    switch (parameterId) {
	    case PARAMETER_ADDRESS_DATA_CHANNEL_TYPE_PROFILE:
		canDataBuffer.length = 8;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;
		canDataBuffer.UI8[4] = (unsigned char) (parameters[channelIndex].chan.type);
		canDataBuffer.UI8[5] = (unsigned char) (parameters[channelIndex].chan.type >> 8);
		canDataBuffer.UI8[6] = (unsigned char) (parameters[channelIndex].chan.profile);
		canDataBuffer.UI8[7] = (unsigned char) (parameters[channelIndex].chan.profile >> 8);

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    case PARAMETER_ADDRESS_DATA_CHANNEL_DESCRIPTOR:
		canDataBuffer.length = 4;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_FAILURE;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    case PARAMETER_ADDRESS_DATA_CHANNEL_TRANSFER_TYPE:
		canDataBuffer.length = 5;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;
		canDataBuffer.UI8[4] = (unsigned char)
		    parameters[channelIndex].transfer_type;

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    case PARAMETER_ADDRESS_DATA_CHANNEL_TRANSFER_ENABLE_FLAG:
		canDataBuffer.length = 5;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;
		canDataBuffer.UI8[4] = parameters[channelIndex].chan.transfer_enable_flag;

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    case PARAMETER_ADDRESS_OUTPUT_DATA_CHANNEL_NODE_ID:	// also PARAMETER_ADDRESS_INPUT_DATA_CHANNEL_SOURCE_VID_PID
		if (parameters[channelIndex].transfer_type == TRANSFER_TYPE_OUTPUT)	// PARAMETER_ADDRESS_OUTPUT_DATA_CHANNEL_NODE_ID
		{
		    canDataBuffer.length = 5;
		    canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		    canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		    canDataBuffer.UI8[2] = parameterId;
		    canDataBuffer.UI8[3] = channelIndex;
		    canDataBuffer.UI8[4] = parameters[channelIndex].ochan.node_id;
		} else {	// PARAMETER_ADDRESS_INPUT_DATA_CHANNEL_SOURCE_VID_PID
		    canDataBuffer.length = 8;
		    canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		    canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		    canDataBuffer.UI8[2] = parameterId;
		    canDataBuffer.UI8[3] = channelIndex;
		    canDataBuffer.UI8[4] = (unsigned char) (parameters[channelIndex].ichan.source_vid);
		    canDataBuffer.UI8[5] = (unsigned char) (parameters[channelIndex].ichan.source_vid >> 8);
		    canDataBuffer.UI8[6] = (unsigned char) (parameters[channelIndex].ichan.source_pid);
		    canDataBuffer.UI8[7] = (unsigned char) (parameters[channelIndex].ichan.source_pid >> 8);
		}

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;

	    case PARAMETER_ADDRESS_INPUT_DATA_CHANNEL_SOURCE_SN_CHANNEL_INDEX:
		if (parameters[channelIndex].transfer_type == TRANSFER_TYPE_INPUT) {
		    canDataBuffer.length = 7;
		    canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		    canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		    canDataBuffer.UI8[2] = parameterId;
		    canDataBuffer.UI8[3] = channelIndex;
		    canDataBuffer.UI8[4] = (unsigned char) (parameters[channelIndex].ichan.source_snum);
		    canDataBuffer.UI8[5] = (unsigned char) (parameters[channelIndex].ichan.source_snum >> 8);
		    canDataBuffer.UI8[6] = (unsigned char) (parameters[channelIndex].ichan.source_channel_index);
		} else {
		    canDataBuffer.length = 4;
		    canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		    canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_FAILURE;
		    canDataBuffer.UI8[2] = parameterId;
		    canDataBuffer.UI8[3] = channelIndex;
		}

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    case PARAMETER_ADDRESS_INPUT_DATA_CHANNEL_SOURCE_NODE_IDS:
		if (parameters[channelIndex].transfer_type == TRANSFER_TYPE_INPUT) {
		    canDataBuffer.length = 6;
		    canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		    canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		    canDataBuffer.UI8[2] = parameterId;
		    canDataBuffer.UI8[3] = channelIndex;
		    canDataBuffer.UI8[4] = (unsigned char) (parameters[channelIndex].ichan.source_device_node_id);
		    canDataBuffer.UI8[5] = (unsigned char) (parameters[channelIndex].ichan.source_channel_node_id);
		} else {
		    canDataBuffer.length = 4;
		    canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		    canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_FAILURE;
		    canDataBuffer.UI8[2] = parameterId;
		    canDataBuffer.UI8[3] = channelIndex;
		}

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;


	    case PARAMETER_ADDRESS_INPUT_DATA_CHANNEL_SOURCE_TYPE_PROFILE:
		if (parameters[channelIndex].transfer_type == TRANSFER_TYPE_INPUT) {
		    canDataBuffer.length = 8;
		    canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		    canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		    canDataBuffer.UI8[2] = parameterId;
		    canDataBuffer.UI8[3] = channelIndex;
		    canDataBuffer.UI8[4] = (unsigned char) (parameters[channelIndex].ichan.source_type);
		    canDataBuffer.UI8[5] = (unsigned char) (parameters[channelIndex].ichan.source_type >> 8);
		    canDataBuffer.UI8[6] = (unsigned char) (parameters[channelIndex].ichan.source_profile);
		    canDataBuffer.UI8[7] = (unsigned char) (parameters[channelIndex].ichan.source_profile >> 8);
		} else {
		    canDataBuffer.length = 4;
		    canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		    canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_FAILURE;
		    canDataBuffer.UI8[2] = parameterId;
		    canDataBuffer.UI8[3] = channelIndex;
		}

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
//#endif
	    case PARAMETER_ADDRESS_DATA_CHANNEL_REQUIRED_INPUT_CHANNELS:
		canDataBuffer.length = 5;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;
		canDataBuffer.UI8[4] = (unsigned char) (parameters[channelIndex].chan.required_input_channels);

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;

	    case PARAMETER_ADDRESS_DATA_CHANNEL_DESIRED_INPUT_CHANNELS:
		canDataBuffer.length = 5;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;
		canDataBuffer.UI8[4] = (unsigned char) (parameters[channelIndex].chan.desired_input_channels);

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;

            case PARAMETER_ADDRESS_INPUT_DATA_CHANNEL_TERMINAL_PROFILE:
		canDataBuffer.length = 6;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;
		canDataBuffer.UI8[4] = (unsigned char) (parameters[channelIndex].ichan.terminal_profile);
		canDataBuffer.UI8[5] = (unsigned char) (parameters[channelIndex].ichan.terminal_profile >> 8);

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;

	    default:
		canDataBuffer.length = 4;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_GET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_FAILURE;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    }
	}

	break;
    case PDCP_COMMAND_SET_DEVICE_PARAMETER:
	parameterId = incomingMessage.data8[1];
	channelIndex = incomingMessage.data8[2];

	if (channelIndex == DEVICE_CHANNEL_INDEX)	// if channelIndex is zero, it's a device-specific parameter...
	{
	    switch (parameterId) {
	    default:
		canDataBuffer.length = 4;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_SET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_FAILURE;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    }
	} else			//if (channelIndex >= 1)
	{
	    switch (parameterId) {
	    case PARAMETER_ADDRESS_DATA_CHANNEL_TRANSFER_ENABLE_FLAG:

		parameters[channelIndex].chan.transfer_enable_flag = incomingMessage.data8[3];

		canDataBuffer.length = 5;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_SET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;
		canDataBuffer.UI8[4] = parameters[channelIndex].chan.transfer_enable_flag;

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		if (parameters[channelIndex].chan.transfer_enable_flag == 1) {
		    SendUpdateDataChannelLinkMessage(channelIndex);
		}

		break;

	    case PARAMETER_ADDRESS_OUTPUT_DATA_CHANNEL_NODE_ID:	// also PARAMETER_ADDRESS_INPUT_DATA_CHANNEL_SOURCE_VID_PID
		if (parameters[channelIndex].transfer_type == TRANSFER_TYPE_OUTPUT)	// PARAMETER_ADDRESS_OUTPUT_DATA_CHANNEL_NODE_ID
		{
		    parameters[channelIndex].ochan.node_id = incomingMessage.data8[3];

		    canDataBuffer.length = 5;
		    canDataBuffer.UI8[0] = PDCP_RESPONSE_SET_DEVICE_PARAMETER;
		    canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		    canDataBuffer.UI8[2] = parameterId;
		    canDataBuffer.UI8[3] = channelIndex;
		    canDataBuffer.UI8[4] = parameters[channelIndex].ochan.node_id;
		} else {	// PARAMETER_ADDRESS_INPUT_DATA_CHANNEL_SOURCE_VID_PID
		    parameters[channelIndex].ichan.source_vid = (((unsigned int)
								  incomingMessage.data8[4]) << 8) + incomingMessage.data8[3];
		    parameters[channelIndex].ichan.source_pid = (((unsigned int)
								  incomingMessage.data8[6]) << 8) + incomingMessage.data8[5];

		    parameters[channelIndex].ichan.source_channel_node_id = 0xFF;
		    parameters[channelIndex].ichan.source_device_node_id = 0xFF;

		    canDataBuffer.length = 8;
		    canDataBuffer.UI8[0] = PDCP_RESPONSE_SET_DEVICE_PARAMETER;
		    canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		    canDataBuffer.UI8[2] = parameterId;
		    canDataBuffer.UI8[3] = channelIndex;
		    canDataBuffer.UI8[4] = (unsigned char) (parameters[channelIndex].ichan.source_vid);
		    canDataBuffer.UI8[5] = (unsigned char) (parameters[channelIndex].ichan.source_vid >> 8);
		    canDataBuffer.UI8[6] = (unsigned char) (parameters[channelIndex].ichan.source_pid);
		    canDataBuffer.UI8[7] = (unsigned char) (parameters[channelIndex].ichan.source_pid >> 8);

		    DataEEWrite(parameters[channelIndex].ichan.source_vid, DEE_ADDR_SOURCE_VID + DEE_CHANNEL_OFFSET + (DEE_CHANNEL_SIZE * (channelIndex - 1)));
		    DataEEWrite(parameters[channelIndex].ichan.source_pid, DEE_ADDR_SOURCE_PID + DEE_CHANNEL_OFFSET + (DEE_CHANNEL_SIZE * (channelIndex - 1)));
		}

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;

	    case PARAMETER_ADDRESS_INPUT_DATA_CHANNEL_SOURCE_SN_CHANNEL_INDEX:

		if (parameters[channelIndex].transfer_type == TRANSFER_TYPE_INPUT) {
		    parameters[channelIndex].ichan.source_snum = (((unsigned int)
								   incomingMessage.data8[4]) << 8) + incomingMessage.data8[3];
		    parameters[channelIndex].ichan.source_channel_index = (((unsigned int)
									    incomingMessage.data8[6]) << 8) + incomingMessage.data8[5];

		    parameters[channelIndex].ichan.source_channel_node_id = 0xFF;
		    parameters[channelIndex].ichan.source_device_node_id = 0xFF;

		    canDataBuffer.length = 7;
		    canDataBuffer.UI8[0] = PDCP_RESPONSE_SET_DEVICE_PARAMETER;
		    canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		    canDataBuffer.UI8[2] = parameterId;
		    canDataBuffer.UI8[3] = channelIndex;
		    canDataBuffer.UI8[4] = (unsigned char) (parameters[channelIndex].ichan.source_snum);
		    canDataBuffer.UI8[5] = (unsigned char) (parameters[channelIndex].ichan.source_snum >> 8);
		    canDataBuffer.UI8[6] = (unsigned char) (parameters[channelIndex].ichan.source_channel_index);

		    DataEEWrite(parameters[channelIndex].ichan.source_snum, DEE_ADDR_SOURCE_SN + DEE_CHANNEL_OFFSET + (DEE_CHANNEL_SIZE * (channelIndex - 1)));
		    DataEEWrite(parameters[channelIndex].ichan.source_channel_index, DEE_ADDR_SOURCE_CHANNEL_INDEX + DEE_CHANNEL_OFFSET + (DEE_CHANNEL_SIZE * (channelIndex - 1)));
		} else {
		    canDataBuffer.length = 4;
		    canDataBuffer.UI8[0] = PDCP_RESPONSE_SET_DEVICE_PARAMETER;
		    canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_FAILURE;
		    canDataBuffer.UI8[2] = parameterId;
		    canDataBuffer.UI8[3] = channelIndex;
		}

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

                SendUpdateDataChannelLinkMessage(channelIndex);


		break;
	    case PARAMETER_ADDRESS_INPUT_DATA_CHANNEL_SOURCE_NODE_IDS:

		if (parameters[channelIndex].transfer_type == TRANSFER_TYPE_INPUT) {
		    parameters[channelIndex].ichan.source_device_node_id = incomingMessage.data8[3];
		    parameters[channelIndex].ichan.source_channel_node_id = incomingMessage.data8[4];
		    parameters[channelIndex].chan.transfer_enable_flag = 1;

		    canDataBuffer.length = 6;
		    canDataBuffer.UI8[0] = PDCP_RESPONSE_SET_DEVICE_PARAMETER;
		    canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		    canDataBuffer.UI8[2] = parameterId;
		    canDataBuffer.UI8[3] = channelIndex;
		    canDataBuffer.UI8[4] = parameters[channelIndex].ichan.source_device_node_id;
		    canDataBuffer.UI8[5] = parameters[channelIndex].ichan.source_channel_node_id;
		} else {
		    canDataBuffer.length = 4;
		    canDataBuffer.UI8[0] = PDCP_RESPONSE_SET_DEVICE_PARAMETER;
		    canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_FAILURE;
		    canDataBuffer.UI8[2] = parameterId;
		    canDataBuffer.UI8[3] = channelIndex;
		}

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;

	    case PARAMETER_ADDRESS_INPUT_DATA_CHANNEL_SOURCE_TYPE_PROFILE:

		if (parameters[channelIndex].transfer_type == TRANSFER_TYPE_INPUT) {
		    parameters[channelIndex].ichan.source_type = (incomingMessage.data8[4] << 8) + incomingMessage.data8[3];
		    parameters[channelIndex].ichan.source_profile = (incomingMessage.data8[6] << 8) + incomingMessage.data8[5];
		    parameters[channelIndex].chan.transfer_enable_flag = 1;

		    canDataBuffer.length = 6;
		    canDataBuffer.UI8[0] = PDCP_RESPONSE_SET_DEVICE_PARAMETER;
		    canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;
		    canDataBuffer.UI8[2] = parameterId;
		    canDataBuffer.UI8[3] = channelIndex;
		    canDataBuffer.UI8[4] = (unsigned char) (parameters[channelIndex].ichan.source_type);
		    canDataBuffer.UI8[5] = (unsigned char) (parameters[channelIndex].ichan.source_type >> 8);
		    canDataBuffer.UI8[6] = (unsigned char) (parameters[channelIndex].ichan.source_profile);
		    canDataBuffer.UI8[7] = (unsigned char) (parameters[channelIndex].ichan.source_profile >> 8);

		    DataEEWrite(parameters[channelIndex].ichan.source_type, DEE_ADDR_SOURCE_TYPE + DEE_CHANNEL_OFFSET + (DEE_CHANNEL_SIZE * (channelIndex - 1)));
		    DataEEWrite(parameters[channelIndex].ichan.source_profile, DEE_ADDR_SOURCE_PROFILE + DEE_CHANNEL_OFFSET + (DEE_CHANNEL_SIZE * (channelIndex - 1)));
		} else {
		    canDataBuffer.length = 4;
		    canDataBuffer.UI8[0] = PDCP_RESPONSE_SET_DEVICE_PARAMETER;
		    canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_FAILURE;
		    canDataBuffer.UI8[2] = parameterId;
		    canDataBuffer.UI8[3] = channelIndex;
		}

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
//#endif
	    default:
		canDataBuffer.length = 4;
		canDataBuffer.UI8[0] = PDCP_RESPONSE_SET_DEVICE_PARAMETER;
		canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_FAILURE;
		canDataBuffer.UI8[2] = parameterId;
		canDataBuffer.UI8[3] = channelIndex;

		messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

		CAN1_sendData(messageSID, &canDataBuffer);

		break;
	    }
	}

	break;
    case PDCP_COMMAND_RESET_DEVICE:
	canDataBuffer.length = 2;
	canDataBuffer.UI8[0] = PDCP_RESPONSE_RESET_DEVICE;
	canDataBuffer.UI8[1] = PDCP_RESPONSE_CODE_SUCCESSFUL;

	messageSID = createSID(PDCP_PRIORITY_NORMAL, PDCP_STANDARD_MESSAGE_MODE, parameters[0].device.node_id);

	CAN1_sendData(messageSID, &canDataBuffer);

	while (C1TR01CONbits.TXREQ0) {
	}			// wait until response has been transfered...

	asm("reset");

	break;
    default:
	break;
    }
}

//*******************************************************************************
